@@ -1,11 +1,9 @@
 /*
-   This is the main SLEPc include file (for C and C++).  It is included
-   by all other SLEPc include files, so it almost never has to be
-   specifically included.
+   Include all top-level SLEPc functionality.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -23,11 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-/*
-   Include all top-level SLEPc functionality
-*/
 #include <slepcsvd.h>
-#include <slepcqep.h>
+#include <slepcpep.h>
 #include <slepcnep.h>
 #include <slepcmfn.h>
 
