@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -39,15 +39,15 @@ PetscErrorCode GetExactEigenvalues(PetscInt M,PetscInt N,PetscInt P,PetscInt nco
   ax = PETSC_PI/2/(M+1);
   ay = PETSC_PI/2/(N+1);
   az = PETSC_PI/2/(P+1);
-  n = ceil(PetscPowReal(nconv,0.33333)+1);
-  ierr = PetscMalloc(n*n*n*sizeof(PetscReal),&evals);CHKERRQ(ierr);
+  n = PetscCeilReal(PetscPowReal((PetscReal)nconv,0.33333)+1);
+  ierr = PetscMalloc1(n*n*n,&evals);CHKERRQ(ierr);
   l = 0;
   for (i=1;i<=n;i++) {
-    sx = sin(ax*i);
+    sx = PetscSinReal(ax*i);
     for (j=1;j<=n;j++) {
-      sy = sin(ay*j);
+      sy = PetscSinReal(ay*j);
       for (k=1;k<=n;k++) {
-        sz = sin(az*k);
+        sz = PetscSinReal(az*k);
         evals[l++] = 4.0*(sx*sx+sy*sy+sz*sz);
       }
     }
@@ -118,8 +118,8 @@ int main(int argc,char **argv)
      Compute the operator matrix that defines the eigensystem, Ax=kx
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = DMDACreate3d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,
-                      DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-10,-10,-10,
+  ierr = DMDACreate3d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,
+                      DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,-10,-10,-10,
                       PETSC_DECIDE,PETSC_DECIDE,PETSC_DECIDE,
                       1,1,NULL,NULL,NULL,&da);CHKERRQ(ierr);
 
@@ -128,14 +128,14 @@ int main(int argc,char **argv)
   ierr = PetscPrintf(PETSC_COMM_WORLD," Grid partitioning: %D %D %D\n",m,n,p);CHKERRQ(ierr);
 
   /* create and fill the matrix */
-  ierr = DMCreateMatrix(da,MATAIJ,&A);CHKERRQ(ierr);
+  ierr = DMCreateMatrix(da,&A);CHKERRQ(ierr);
   ierr = FillMatrix(da,A);CHKERRQ(ierr);
 
   /* create random initial vector */
   seed = 1;
-  ierr = PetscOptionsGetInt(NULL,"-seed",&seed,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-seed",&seed,NULL);CHKERRQ(ierr);
   if (seed<0) SETERRQ(PETSC_COMM_WORLD,1,"Seed must be >=0");
-  ierr = MatGetVecs(A,&v0,NULL);CHKERRQ(ierr);
+  ierr = MatCreateVecs(A,&v0,NULL);CHKERRQ(ierr);
   ierr = PetscRandomCreate(PETSC_COMM_WORLD,&rctx);CHKERRQ(ierr);
   ierr = PetscRandomSetFromOptions(rctx);CHKERRQ(ierr);
   for (i=0;i<seed;i++) {   /* simulate different seeds in the random generator */
@@ -189,7 +189,7 @@ int main(int argc,char **argv)
   ierr = EPSGetDimensions(eps,&nev,NULL,NULL);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD," Number of requested eigenvalues: %D\n",nev);CHKERRQ(ierr);
   ierr = EPSGetTolerances(eps,&tol,&maxit);CHKERRQ(ierr);
-  ierr = PetscPrintf(PETSC_COMM_WORLD," Stopping condition: tol=%.4G, maxit=%D\n",tol,maxit);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD," Stopping condition: tol=%.4g, maxit=%D\n",(double)tol,maxit);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                     Display solution and clean up
@@ -202,7 +202,7 @@ int main(int argc,char **argv)
   ierr = PetscPrintf(PETSC_COMM_WORLD," Number of converged approximate eigenpairs: %D\n\n",nconv);CHKERRQ(ierr);
 
   if (nconv>0) {
-    ierr = PetscMalloc(nconv*sizeof(PetscReal),&exact);CHKERRQ(ierr);
+    ierr = PetscMalloc1(nconv,&exact);CHKERRQ(ierr);
     ierr = GetExactEigenvalues(M,N,P,nconv,exact);CHKERRQ(ierr);
     /*
        Display eigenvalues and relative errors
@@ -220,7 +220,7 @@ int main(int argc,char **argv)
       /*
          Compute the relative error associated to each eigenpair
       */
-      ierr = EPSComputeRelativeError(eps,i,&error);CHKERRQ(ierr);
+      ierr = EPSComputeError(eps,i,EPS_ERROR_RELATIVE,&error);CHKERRQ(ierr);
 
 #if defined(PETSC_USE_COMPLEX)
       re = PetscRealPart(kr);
@@ -231,7 +231,7 @@ int main(int argc,char **argv)
 #endif
       if (im!=0.0) SETERRQ(PETSC_COMM_WORLD,1,"Eigenvalue should be real");
       else {
-        ierr = PetscPrintf(PETSC_COMM_WORLD,"   %12G       %12G        %12G\n",re,error,PetscAbsReal(re-exact[i]));CHKERRQ(ierr);
+        ierr = PetscPrintf(PETSC_COMM_WORLD,"   %12g       %12g        %12g\n",(double)re,(double)error,(double)PetscAbsReal(re-exact[i]));CHKERRQ(ierr);
       }
     }
     ierr = PetscFree(exact);CHKERRQ(ierr);
@@ -241,9 +241,9 @@ int main(int argc,char **argv)
   /*
      Show computing times
   */
-  ierr = PetscOptionsHasName(NULL,"-showtimes",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-showtimes",&flg);CHKERRQ(ierr);
   if (flg) {
-    ierr = PetscPrintf(PETSC_COMM_WORLD," Elapsed time: %G (setup), %G (solve)\n",t2-t1,t3-t2);CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_WORLD," Elapsed time: %g (setup), %g (solve)\n",(double)(t2-t1),(double)(t3-t2));CHKERRQ(ierr);
   }
 
   /*
@@ -255,6 +255,6 @@ int main(int argc,char **argv)
   ierr = PetscRandomDestroy(&rctx);CHKERRQ(ierr);
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = SlepcFinalize();
-  return 0;
+  return ierr;
 }
 
