@@ -17,13 +17,11 @@
 
        [2] E. Romero and J.E. Roman, "A parallel implementation of
            Davidson methods for large-scale eigenvalue problems in
-           SLEPc", submitted, 2013.
-
-   Last update: Jul 2012
+           SLEPc", ACM Trans. Math. Software 40(2), Article 13, 2014.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -41,23 +39,23 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/epsimpl.h>                /*I "slepceps.h" I*/
-#include <../src/eps/impls/davidson/common/davidson.h>
+#include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <../src/eps/impls/davidson/davidson.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_JD"
-PetscErrorCode EPSSetFromOptions_JD(EPS eps)
+PetscErrorCode EPSSetFromOptions_JD(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
   PetscBool      flg,op;
   PetscInt       opi,opi0;
   PetscReal      opf;
   KSP            ksp;
-  EPSOrthType    orth;
-  const char     *orth_list[3] = {"I","B","B_opt"};
+  PetscBool      orth;
+  const char     *orth_list[2] = {"I","B"};
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("EPS Jacobi-Davidson (JD) Options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"EPS Jacobi-Davidson (JD) Options");CHKERRQ(ierr);
 
   ierr = EPSJDGetKrylovStart(eps,&op);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-eps_jd_krylov_start","Start the searching subspace with a krylov basis","EPSJDSetKrylovStart",op,&op,&flg);CHKERRQ(ierr);
@@ -83,8 +81,8 @@ PetscErrorCode EPSSetFromOptions_JD(EPS eps)
   if (flg) { ierr = EPSJDSetFix(eps,opf);CHKERRQ(ierr); }
 
   ierr = EPSJDGetBOrth(eps,&orth);CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-eps_jd_borth","orthogonalization used in the search subspace","EPSJDSetBOrth",orth_list,3,orth_list[orth-1],&opi,&flg);CHKERRQ(ierr);
-  if (flg) { ierr = EPSJDSetBOrth(eps,(EPSOrthType)(opi+1));CHKERRQ(ierr); }
+  ierr = PetscOptionsEList("-eps_jd_borth","orthogonalization used in the search subspace","EPSJDSetBOrth",orth_list,2,orth_list[orth?1:0],&opi,&flg);CHKERRQ(ierr);
+  if (flg) { ierr = EPSJDSetBOrth(eps,opi==1?PETSC_TRUE:PETSC_FALSE);CHKERRQ(ierr); }
 
   ierr = EPSJDGetConstCorrectionTol(eps,&op);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-eps_jd_const_correction_tol","Disable the dynamic stopping criterion when solving the correction equation","EPSJDSetConstCorrectionTol",op,&op,&flg);CHKERRQ(ierr);
@@ -138,6 +136,39 @@ PetscErrorCode EPSSetUp_JD(EPS eps)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "EPSView_JD"
+PetscErrorCode EPSView_JD(EPS eps,PetscViewer viewer)
+{
+  PetscErrorCode ierr;
+  PetscBool      isascii,opb;
+  PetscInt       opi,opi0;
+  PetscBool      borth;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    ierr = EPSXDGetBOrth_XD(eps,&borth);CHKERRQ(ierr);
+    if (borth) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  JD: search subspace is B-orthogonalized\n");CHKERRQ(ierr);
+    } else {
+      ierr = PetscViewerASCIIPrintf(viewer,"  JD: search subspace is orthogonalized\n");CHKERRQ(ierr);
+    }
+    ierr = EPSXDGetBlockSize_XD(eps,&opi);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  JD: block size=%D\n",opi);CHKERRQ(ierr);
+    ierr = EPSXDGetKrylovStart_XD(eps,&opb);CHKERRQ(ierr);
+    if (!opb) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  JD: type of the initial subspace: non-Krylov\n");CHKERRQ(ierr);
+    } else {
+      ierr = PetscViewerASCIIPrintf(viewer,"  JD: type of the initial subspace: Krylov\n");CHKERRQ(ierr);
+    }
+    ierr = EPSXDGetRestart_XD(eps,&opi,&opi0);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  JD: size of the subspace after restarting: %D\n",opi);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  JD: number of vectors after restarting from the previous iteration: %D\n",opi0);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_JD"
 PetscErrorCode EPSDestroy_JD(EPS eps)
@@ -222,7 +253,7 @@ PetscErrorCode EPSJDGetKrylovStart(EPS eps,PetscBool *krylovstart)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(krylovstart,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetKrylovStart_C",(EPS,PetscBool*),(eps,krylovstart));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetKrylovStart_C",(EPS,PetscBool*),(eps,krylovstart));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -281,7 +312,7 @@ PetscErrorCode EPSJDGetBlockSize(EPS eps,PetscInt *blocksize)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidIntPointer(blocksize,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetBlockSize_C",(EPS,PetscInt*),(eps,blocksize));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetBlockSize_C",(EPS,PetscInt*),(eps,blocksize));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -310,7 +341,7 @@ PetscErrorCode EPSJDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = PetscTryMethod(eps,"EPSJDGetRestart_C",(EPS,PetscInt*,PetscInt*),(eps,minv,plusk));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetRestart_C",(EPS,PetscInt*,PetscInt*),(eps,minv,plusk));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -379,7 +410,7 @@ PetscErrorCode EPSJDGetInitialSize(EPS eps,PetscInt *initialsize)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidIntPointer(initialsize,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetInitialSize_C",(EPS,PetscInt*),(eps,initialsize));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetInitialSize_C",(EPS,PetscInt*),(eps,initialsize));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -450,7 +481,7 @@ PetscErrorCode EPSJDGetFix(EPS eps,PetscReal *fix)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(fix,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetFix_C",(EPS,PetscReal*),(eps,fix));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetFix_C",(EPS,PetscReal*),(eps,fix));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -546,7 +577,7 @@ PetscErrorCode EPSJDGetConstCorrectionTol(EPS eps,PetscBool *constant)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(constant,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetConstCorrectionTol",(EPS,PetscBool*),(eps,constant));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetConstCorrectionTol_C",(EPS,PetscBool*),(eps,constant));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -576,7 +607,7 @@ PetscErrorCode EPSJDGetWindowSizes(EPS eps,PetscInt *pwindow,PetscInt *qwindow)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = PetscTryMethod(eps,"EPSJDGetWindowSizes_C",(EPS,PetscInt*,PetscInt*),(eps,pwindow,qwindow));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetWindowSizes_C",(EPS,PetscInt*,PetscInt*),(eps,pwindow,qwindow));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -624,36 +655,23 @@ PetscErrorCode EPSJDSetWindowSizes(EPS eps,PetscInt pwindow,PetscInt qwindow)
 
    Input Parameters:
 +  eps   - the eigenproblem solver context
--  borth - the kind of orthogonalization
-
-   Possible values:
-   The parameter 'borth' can have one of these values
-
-+   EPS_ORTH_I - orthogonalization of the search subspace
-.   EPS_ORTH_B - B-orthogonalization of the search subspace
--   EPS_ORTH_BOPT - B-orthogonalization of the search subspace with an alternative method
+-  borth - whether to B-orthogonalize the search subspace
 
    Options Database Key:
 .  -eps_jd_borth - Set the orthogonalization used in the search subspace
 
-   Notes:
-   If borth is EPS_ORTH_B, the solver uses a variant of Gram-Schmidt (selected in
-   IP associated to the EPS) with the inner product defined by the matrix problem B.
-   If borth is EPS_ORTH_BOPT, it uses another variant of Gram-Schmidt that only performs
-   one matrix-vector product although more than one reorthogonalization would be done.
-
    Level: advanced
 
 .seealso: EPSJDGetBOrth()
 @*/
-PetscErrorCode EPSJDSetBOrth(EPS eps,EPSOrthType borth)
+PetscErrorCode EPSJDSetBOrth(EPS eps,PetscBool borth)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveBool(eps,borth,2);
-  ierr = PetscTryMethod(eps,"EPSJDSetBOrth_C",(EPS,EPSOrthType),(eps,borth));CHKERRQ(ierr);
+  ierr = PetscTryMethod(eps,"EPSJDSetBOrth_C",(EPS,PetscBool),(eps,borth));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -669,23 +687,20 @@ PetscErrorCode EPSJDSetBOrth(EPS eps,EPSOrthType borth)
 .  eps - the eigenproblem solver context
 
    Output Parameters:
-.  borth - the kind of orthogonalization
-
-   Notes:
-   See EPSJDSetBOrth() for possible values of 'borth'.
+.  borth - whether to B-orthogonalize the search subspace
 
    Level: advanced
 
-.seealso: EPSJDSetBOrth(), EPSOrthType
+.seealso: EPSJDSetBOrth()
 @*/
-PetscErrorCode EPSJDGetBOrth(EPS eps,EPSOrthType *borth)
+PetscErrorCode EPSJDGetBOrth(EPS eps,PetscBool *borth)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(borth,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetBOrth_C",(EPS,EPSOrthType*),(eps,borth));CHKERRQ(ierr);
+  ierr = PetscUseMethod(eps,"EPSJDGetBOrth_C",(EPS,PetscBool*),(eps,borth));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -694,13 +709,29 @@ PetscErrorCode EPSJDGetBOrth(EPS eps,EPSOrthType *borth)
 PETSC_EXTERN PetscErrorCode EPSCreate_JD(EPS eps)
 {
   PetscErrorCode ierr;
+  EPS_DAVIDSON   *data;
 
   PetscFunctionBegin;
-  /* Load the Davidson solver */
-  ierr = EPSCreate_XD(eps);CHKERRQ(ierr);
-  ierr = EPSXDSetMethod(eps,DVD_METH_JD);CHKERRQ(ierr);
+  ierr = PetscNewLog(eps,&data);CHKERRQ(ierr);
+  eps->data = (void*)data;
 
-  /* Overload the JD properties */
+  data->blocksize   = 1;
+  data->initialsize = 6;
+  data->minv        = 6;
+  data->plusk       = 0;
+  data->ipB         = PETSC_TRUE;
+  data->fix         = 0.01;
+  data->krylovstart = PETSC_FALSE;
+  data->dynamic     = PETSC_FALSE;
+  data->cX_in_proj  = 0;
+  data->cX_in_impr  = 0;
+
+  eps->ops->solve          = EPSSolve_XD;
+  eps->ops->setup          = EPSSetUp_XD;
+  eps->ops->reset          = EPSReset_XD;
+  eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->computevectors = EPSComputeVectors_XD;
+  eps->ops->view           = EPSView_JD;
   eps->ops->setfromoptions = EPSSetFromOptions_JD;
   eps->ops->setup          = EPSSetUp_JD;
   eps->ops->destroy        = EPSDestroy_JD;
