@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -21,8 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/epsimpl.h>        /*I "slepceps.h" I*/
-#include <slepc-private/stimpl.h>         /*I "slepcst.h" I*/
+#include <slepc/private/epsimpl.h>
 #include <../src/eps/impls/external/arpack/arpackp.h>
 
 PetscErrorCode EPSSolve_ARPACK(EPS);
@@ -33,6 +32,7 @@ PetscErrorCode EPSSetUp_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       ncv;
+  PetscBool      flg,istrivial;
   EPS_ARPACK     *ar = (EPS_ARPACK*)eps->data;
 
   PetscFunctionBegin;
@@ -46,42 +46,48 @@ PetscErrorCode EPSSetUp_ARPACK(EPS eps)
   ncv = eps->ncv;
 #if defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(ar->rwork);CHKERRQ(ierr);
-  ierr = PetscMalloc(ncv*sizeof(PetscReal),&ar->rwork);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,ncv*sizeof(PetscReal));CHKERRQ(ierr);
+  ierr = PetscMalloc1(ncv,&ar->rwork);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,ncv*sizeof(PetscReal));CHKERRQ(ierr);
   ierr = PetscBLASIntCast(3*ncv*ncv+5*ncv,&ar->lworkl);CHKERRQ(ierr);
   ierr = PetscFree(ar->workev);CHKERRQ(ierr);
-  ierr = PetscMalloc(3*ncv*sizeof(PetscScalar),&ar->workev);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,3*ncv*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMalloc1(3*ncv,&ar->workev);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,3*ncv*sizeof(PetscScalar));CHKERRQ(ierr);
 #else
   if (eps->ishermitian) {
     ierr = PetscBLASIntCast(ncv*(ncv+8),&ar->lworkl);CHKERRQ(ierr);
   } else {
     ierr = PetscBLASIntCast(3*ncv*ncv+6*ncv,&ar->lworkl);CHKERRQ(ierr);
     ierr = PetscFree(ar->workev);CHKERRQ(ierr);
-    ierr = PetscMalloc(3*ncv*sizeof(PetscScalar),&ar->workev);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory(eps,3*ncv*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscMalloc1(3*ncv,&ar->workev);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)eps,3*ncv*sizeof(PetscScalar));CHKERRQ(ierr);
   }
 #endif
   ierr = PetscFree(ar->workl);CHKERRQ(ierr);
-  ierr = PetscMalloc(ar->lworkl*sizeof(PetscScalar),&ar->workl);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,ar->lworkl*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMalloc1(ar->lworkl,&ar->workl);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,ar->lworkl*sizeof(PetscScalar));CHKERRQ(ierr);
   ierr = PetscFree(ar->select);CHKERRQ(ierr);
-  ierr = PetscMalloc(ncv*sizeof(PetscBool),&ar->select);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,ncv*sizeof(PetscBool));CHKERRQ(ierr);
+  ierr = PetscMalloc1(ncv,&ar->select);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,ncv*sizeof(PetscBool));CHKERRQ(ierr);
   ierr = PetscFree(ar->workd);CHKERRQ(ierr);
-  ierr = PetscMalloc(3*eps->nloc*sizeof(PetscScalar),&ar->workd);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,3*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMalloc1(3*eps->nloc,&ar->workd);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,3*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
 
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   if (eps->balance!=EPS_BALANCE_NONE) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Balancing not supported in the Arpack interface");
   if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
+  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"External packages do not support user-defined stopping test");
 
-  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
+  ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
+  ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
   ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
 
+  ierr = PetscObjectTypeCompare((PetscObject)eps->V,BVVECS,&flg);CHKERRQ(ierr);
+  if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires a BV with contiguous storage");
+  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
+  if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
+
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_ARPACK;
   PetscFunctionReturn(0);
 }
@@ -99,7 +105,7 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   PetscBLASInt   fcomm;
 #endif
   PetscScalar    sigmar,*pV,*resid;
-  Vec            x,y,w = eps->work[0];
+  Vec            v0,x,y,w = eps->work[0];
   Mat            A;
   PetscBool      isSinv,isShift,rvec;
 #if !defined(PETSC_USE_COMPLEX)
@@ -115,8 +121,11 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   ierr = PetscBLASIntCast(eps->nloc,&n);CHKERRQ(ierr);
   ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&x);CHKERRQ(ierr);
   ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&y);CHKERRQ(ierr);
-  ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
-  ierr = EPSGetStartVector(eps,0,eps->work[1],NULL);CHKERRQ(ierr);
+  ierr = EPSGetStartVector(eps,0,NULL);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(eps->V,0,0);CHKERRQ(ierr);  /* just for deflation space */
+  ierr = BVGetColumn(eps->V,0,&v0);CHKERRQ(ierr);
+  ierr = VecCopy(v0,eps->work[1]);CHKERRQ(ierr);
+  ierr = VecGetArray(v0,&pV);CHKERRQ(ierr);
   ierr = VecGetArray(eps->work[1],&resid);CHKERRQ(ierr);
 
   ido  = 0;            /* first call to reverse communication interface */
@@ -214,25 +223,25 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
         ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
       } else if (ido == 2) {
         /* Y = B * X */
-        ierr = IPApplyMatrix(eps->ip,x,y);CHKERRQ(ierr);
+        ierr = BVApplyMatrix(eps->V,x,y);CHKERRQ(ierr);
       } else { /* ido == 1 */
         if (iparam[6] == 3 && bmat[0] == 'G') {
           /* Y = OP * X for shift-and-invert with B semi-positive definite */
-          ierr = STMatSolve(eps->st,1,x,y);CHKERRQ(ierr);
+          ierr = STMatSolve(eps->st,x,y);CHKERRQ(ierr);
         } else if (iparam[6] == 2) {
           /* X=A*X Y=B^-1*X for shift with B positive definite */
           ierr = MatMult(A,x,y);CHKERRQ(ierr);
           if (sigmar != 0.0) {
-            ierr = IPApplyMatrix(eps->ip,x,w);CHKERRQ(ierr);
+            ierr = BVApplyMatrix(eps->V,x,w);CHKERRQ(ierr);
             ierr = VecAXPY(y,sigmar,w);CHKERRQ(ierr);
           }
           ierr = VecCopy(y,x);CHKERRQ(ierr);
-          ierr = STMatSolve(eps->st,1,x,y);CHKERRQ(ierr);
+          ierr = STMatSolve(eps->st,x,y);CHKERRQ(ierr);
         } else {
           /* Y = OP * X */
           ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
         }
-        ierr = IPOrthogonalize(eps->ip,0,NULL,eps->nds,NULL,eps->defl,y,NULL,NULL,NULL);CHKERRQ(ierr);
+        ierr = BVOrthogonalizeVec(eps->V,y,NULL,NULL,NULL);CHKERRQ(ierr);
       }
 
       ierr = VecResetArray(x);CHKERRQ(ierr);
@@ -265,7 +274,8 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
     if (info!=0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error reported by ARPACK subroutine xxEUPD (%d)",info);
   }
 
-  ierr = VecRestoreArray(eps->V[0],&pV);CHKERRQ(ierr);
+  ierr = VecRestoreArray(v0,&pV);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(eps->V,0,&v0);CHKERRQ(ierr);
   ierr = VecRestoreArray(eps->work[1],&resid);CHKERRQ(ierr);
   if (eps->nconv >= eps->nev) eps->reason = EPS_CONVERGED_TOL;
   else eps->reason = EPS_DIVERGED_ITS;
@@ -311,7 +321,6 @@ PetscErrorCode EPSReset_ARPACK(EPS eps)
 #if defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(ar->rwork);CHKERRQ(ierr);
 #endif
-  ierr = EPSReset_Default(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -330,15 +339,17 @@ PetscErrorCode EPSDestroy_ARPACK(EPS eps)
 #define __FUNCT__ "EPSCreate_ARPACK"
 PETSC_EXTERN PetscErrorCode EPSCreate_ARPACK(EPS eps)
 {
+  EPS_ARPACK     *ctx;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(eps,EPS_ARPACK,&eps->data);CHKERRQ(ierr);
+  ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
+  eps->data = (void*)ctx;
+
   eps->ops->setup                = EPSSetUp_ARPACK;
   eps->ops->destroy              = EPSDestroy_ARPACK;
   eps->ops->reset                = EPSReset_ARPACK;
   eps->ops->backtransform        = EPSBackTransform_ARPACK;
-  eps->ops->computevectors       = EPSComputeVectors_Default;
   PetscFunctionReturn(0);
 }
 
