@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -21,8 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/epsimpl.h>    /*I "slepceps.h" I*/
-#include <slepc-private/stimpl.h>
+#include <slepc/private/epsimpl.h>    /*I "slepceps.h" I*/
 
 PetscErrorCode EPSSolve_PRIMME(EPS);
 
@@ -40,31 +39,13 @@ typedef struct {
   PetscReal target;               /* a copy of eps's target */
 } EPS_PRIMME;
 
-EPSPRIMMEMethod methodN[] = {
-  EPS_PRIMME_DYNAMIC,
-  EPS_PRIMME_DEFAULT_MIN_TIME,
-  EPS_PRIMME_DEFAULT_MIN_MATVECS,
-  EPS_PRIMME_ARNOLDI,
-  EPS_PRIMME_GD,
-  EPS_PRIMME_GD_PLUSK,
-  EPS_PRIMME_GD_OLSEN_PLUSK,
-  EPS_PRIMME_JD_OLSEN_PLUSK,
-  EPS_PRIMME_RQI,
-  EPS_PRIMME_JDQR,
-  EPS_PRIMME_JDQMR,
-  EPS_PRIMME_JDQMR_ETOL,
-  EPS_PRIMME_SUBSPACE_ITERATION,
-  EPS_PRIMME_LOBPCG_ORTHOBASIS,
-  EPS_PRIMME_LOBPCG_ORTHOBASISW
-};
-
 static void multMatvec_PRIMME(void *in,void *out,int *blockSize,primme_params *primme);
 static void applyPreconditioner_PRIMME(void *in,void *out,int *blockSize,struct primme_params *primme);
 
 static void par_GlobalSumDouble(void *sendBuf,void *recvBuf,int *count,primme_params *primme)
 {
   PetscErrorCode ierr;
-  ierr = MPI_Allreduce((double*)sendBuf,(double*)recvBuf,*count,MPI_DOUBLE,MPI_SUM,primme->commInfo);CHKERRABORT(primme->commInfo,ierr);
+  ierr = MPI_Allreduce((double*)sendBuf,(double*)recvBuf,*count,MPI_DOUBLE,MPI_SUM,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),ierr);
 }
 
 #undef __FUNCT__
@@ -75,7 +56,7 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   PetscMPIInt    numProcs,procID;
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
   primme_params  *primme = &ops->primme;
-  PetscBool      t;
+  PetscBool      istrivial,flg;
 
   PetscFunctionBegin;
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)eps),&numProcs);CHKERRQ(ierr);
@@ -84,31 +65,25 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   /* Check some constraints and set some default values */
   if (!eps->max_it) eps->max_it = PetscMax(1000,eps->n);
   ierr = STGetOperators(eps->st,0,&ops->A);CHKERRQ(ierr);
-  if (!ops->A) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"The problem matrix has to be specified first");
   if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME is only available for Hermitian problems");
   if (eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME is not available for generalized problems");
   if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
+  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"External packages do not support user-defined stopping test");
   if (!eps->which) eps->which = EPS_LARGEST_REAL;
   if (eps->converged != EPSConvergedAbsolute) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only supports absolute convergence test");
-
-  /* Change the default sigma to inf if necessary */
-  if (eps->which == EPS_LARGEST_MAGNITUDE || eps->which == EPS_LARGEST_REAL || eps->which == EPS_LARGEST_IMAGINARY) {
-    ierr = STSetDefaultShift(eps->st,3e300);CHKERRQ(ierr);
-  }
-
-  /* Avoid setting the automatic shift when a target is set */
-  ierr = STSetDefaultShift(eps->st,0.0);CHKERRQ(ierr);
+  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
+  if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
 
   ierr = STSetUp(eps->st);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&t);CHKERRQ(ierr);
-  if (!t) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only works with STPRECOND");
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&flg);CHKERRQ(ierr);
+  if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only works with STPRECOND");
 
   /* Transfer SLEPc options to PRIMME options */
   primme->n          = eps->n;
   primme->nLocal     = eps->nloc;
   primme->numEvals   = eps->nev;
   primme->matrix     = ops;
-  primme->commInfo   = PetscObjectComm((PetscObject)eps);
+  primme->commInfo   = eps;
   primme->maxMatvecs = eps->max_it;
   primme->eps        = eps->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:eps->tol;
   primme->numProcs   = numProcs;
@@ -116,7 +91,6 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   primme->printLevel = 0;
   primme->correctionParams.precondition = 1;
 
-  if (!eps->which) eps->which = EPS_LARGEST_REAL;
   switch (eps->which) {
     case EPS_LARGEST_REAL:
       primme->target = primme_largest;
@@ -132,7 +106,7 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
       primme->targetShifts = &ops->target;
       break;
     default:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"'which' value does not supported by PRIMME");
+      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"'which' value not supported by PRIMME");
       break;
   }
 
@@ -147,14 +121,16 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   /* Set workspace */
-  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
+  ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->V,BVVECS,&flg);CHKERRQ(ierr);
+  if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires a BV with contiguous storage");
 
   /* Setup the preconditioner */
   ops->eps = eps;
   if (primme->correctionParams.precondition) {
     ierr = STGetKSP(eps->st,&ops->ksp);CHKERRQ(ierr);
-    ierr = PetscObjectTypeCompare((PetscObject)ops->ksp,KSPPREONLY,&t);CHKERRQ(ierr);
-    if (!t) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only works with KSPPREONLY");
+    ierr = PetscObjectTypeCompare((PetscObject)ops->ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only works with KSPPREONLY");
     primme->preconditioner = NULL;
     primme->applyPreconditioner = applyPreconditioner_PRIMME;
   }
@@ -162,11 +138,10 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   /* Prepare auxiliary vectors */
   ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,eps->n,NULL,&ops->x);CHKERRQ(ierr);
   ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,eps->n,NULL,&ops->y);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(eps,ops->x);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(eps,ops->y);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->x);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->y);CHKERRQ(ierr);
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_PRIMME;
   PetscFunctionReturn(0);
 }
@@ -178,6 +153,7 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   PetscErrorCode ierr;
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
   PetscScalar    *a;
+  Vec            v0;
 #if defined(PETSC_USE_COMPLEX)
   PetscInt       i;
   PetscReal      *evals;
@@ -190,40 +166,25 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   ops->primme.iseed[0] = -1;
 
   /* Call PRIMME solver */
-  ierr = VecGetArray(eps->V[0],&a);CHKERRQ(ierr);
+  ierr = BVGetColumn(eps->V,0,&v0);CHKERRQ(ierr);
+  ierr = VecGetArray(v0,&a);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = dprimme(eps->eigr,a,eps->errest,&ops->primme);CHKERRQ(ierr);
+  ierr = dprimme(eps->eigr,a,eps->errest,&ops->primme);
+  if (ierr) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%d",ierr);
 #else
   /* PRIMME returns real eigenvalues, but SLEPc works with complex ones */
-  ierr = PetscMalloc(eps->ncv*sizeof(PetscReal),&evals);CHKERRQ(ierr);
-  ierr = zprimme(evals,(Complex_Z*)a,eps->errest,&ops->primme);CHKERRQ(ierr);
-  for (i=0;i<eps->ncv;i++)
-    eps->eigr[i] = evals[i];
+  ierr = PetscMalloc1(eps->ncv,&evals);CHKERRQ(ierr);
+  ierr = zprimme(evals,(Complex_Z*)a,eps->errest,&ops->primme);
+  if (ierr) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%d",ierr);
+  for (i=0;i<eps->ncv;i++) eps->eigr[i] = evals[i];
   ierr = PetscFree(evals);CHKERRQ(ierr);
 #endif
-  ierr = VecRestoreArray(eps->V[0],&a);CHKERRQ(ierr);
+  ierr = VecRestoreArray(v0,&a);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(eps->V,0,&v0);CHKERRQ(ierr);
 
-  switch (ierr) {
-    case 0: /* Successful */
-      break;
-    case -1:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME: Failed to open output file");
-      break;
-    case -2:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME: Insufficient integer or real workspace allocated");
-      break;
-    case -3:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME: main_iter encountered a problem");
-      break;
-    default:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME: some parameters wrong configured");
-      break;
-  }
-
-  eps->nconv      = ops->primme.initSize >= 0 ? ops->primme.initSize : 0;
-  eps->reason     = eps->ncv >= eps->nev ? EPS_CONVERGED_TOL: EPS_DIVERGED_ITS;
-  eps->its        = ops->primme.stats.numOuterIterations;
-  eps->st->applys = ops->primme.stats.numMatvecs;
+  eps->nconv  = ops->primme.initSize >= 0 ? ops->primme.initSize : 0;
+  eps->reason = eps->ncv >= eps->nev ? EPS_CONVERGED_TOL: EPS_DIVERGED_ITS;
+  eps->its    = ops->primme.stats.numOuterIterations;
   PetscFunctionReturn(0);
 }
 
@@ -256,7 +217,7 @@ static void multMatvec_PRIMME(void *in,void *out,int *blockSize,primme_params *p
 static void applyPreconditioner_PRIMME(void *in,void *out,int *blockSize,struct primme_params *primme)
 {
   PetscErrorCode ierr;
-  PetscInt       i,N = primme->n,lits;
+  PetscInt       i,N = primme->n;
   EPS_PRIMME     *ops = (EPS_PRIMME*)primme->matrix;
   Vec            x = ops->x,y = ops->y;
 
@@ -265,11 +226,7 @@ static void applyPreconditioner_PRIMME(void *in,void *out,int *blockSize,struct
     /* build vectors using 'in' an 'out' workspace */
     ierr = VecPlaceArray(x,(PetscScalar*)in+N*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
     ierr = VecPlaceArray(y,(PetscScalar*)out+N*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-
     ierr = KSPSolve(ops->ksp,x,y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ierr = KSPGetIterationNumber(ops->ksp,&lits);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ops->eps->st->lineariterations+= lits;
-
     ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
     ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
   }
@@ -287,7 +244,6 @@ PetscErrorCode EPSReset_PRIMME(EPS eps)
   primme_Free(&ops->primme);
   ierr = VecDestroy(&ops->x);CHKERRQ(ierr);
   ierr = VecDestroy(&ops->y);CHKERRQ(ierr);
-  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -319,7 +275,7 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  PRIMME: block size=%d\n",primme->maxBlockSize);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  PRIMME: block size=%D\n",primme->maxBlockSize);CHKERRQ(ierr);
     ierr = EPSPRIMMEGetMethod(eps,&methodn);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  PRIMME: solver method: %s\n",EPSPRIMMEMethods[methodn]);CHKERRQ(ierr);
 
@@ -332,7 +288,7 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_PRIMME"
-PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps)
+PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode  ierr;
   EPS_PRIMME      *ctx = (EPS_PRIMME*)eps->data;
@@ -342,7 +298,7 @@ PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps)
   KSP             ksp;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("EPS PRIMME Options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"EPS PRIMME Options");CHKERRQ(ierr);
   ierr = PetscOptionsInt("-eps_primme_block_size","Maximum block size","EPSPRIMMESetBlockSize",ctx->primme.maxBlockSize,&bs,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = EPSPRIMMESetBlockSize(eps,bs);CHKERRQ(ierr);
@@ -383,14 +339,9 @@ static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
 #undef __FUNCT__
 #define __FUNCT__ "EPSPRIMMESetBlockSize"
 /*@
-   EPSPRIMMESetBlockSize - The maximum block size the code will try to use.
-   The user should set
-   this based on the architecture specifics of the target computer,
-   as well as any a priori knowledge of multiplicities. The code does
-   NOT require BlockSize > 1 to find multiple eigenvalues.  For some
-   methods, keeping BlockSize = 1 yields the best overall performance.
+   EPSPRIMMESetBlockSize - The maximum block size that PRIMME will try to use.
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -403,7 +354,13 @@ static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
    If the block size is not set, the value established by primme_initialize
    is used.
 
+   The user should set the block size based on the architecture specifics
+   of the target computer, as well as any a priori knowledge of multiplicities.
+   The code does NOT require bs > 1 to find multiple eigenvalues. For some
+   methods, keeping bs = 1 yields the best overall performance.
+
    Level: advanced
+
 .seealso: EPSPRIMMEGetBlockSize()
 @*/
 PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs)
@@ -424,7 +381,7 @@ static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
 
   PetscFunctionBegin;
-  if (bs) *bs = ops->primme.maxBlockSize;
+  *bs = ops->primme.maxBlockSize;
   PetscFunctionReturn(0);
 }
 
@@ -433,15 +390,16 @@ static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
 /*@
    EPSPRIMMEGetBlockSize - Get the maximum block size the code will try to use.
 
-   Collective on EPS
+   Not Collective
 
-   Input Parameters:
+   Input Parameter:
 .  eps - the eigenproblem solver context
 
-   Output Parameters:
+   Output Parameter:
 .  bs - returned block size
 
    Level: advanced
+
 .seealso: EPSPRIMMESetBlockSize()
 @*/
 PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs)
@@ -450,7 +408,8 @@ PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = PetscTryMethod(eps,"EPSPRIMMEGetBlockSize_C",(EPS,PetscInt*),(eps,bs));CHKERRQ(ierr);
+  PetscValidPointer(bs,2);
+  ierr = PetscUseMethod(eps,"EPSPRIMMEGetBlockSize_C",(EPS,PetscInt*),(eps,bs));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -461,8 +420,7 @@ static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
 
   PetscFunctionBegin;
-  if (method == PETSC_DEFAULT) ops->method = DEFAULT_MIN_TIME;
-  else ops->method = (primme_preset_method)method;
+  ops->method = (primme_preset_method)method;
   PetscFunctionReturn(0);
 }
 
@@ -471,20 +429,14 @@ static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
 /*@
    EPSPRIMMESetMethod - Sets the method for the PRIMME library.
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
--  method - method that will be used by PRIMME. It must be one of:
-    EPS_PRIMME_DYNAMIC, EPS_PRIMME_DEFAULT_MIN_TIME(EPS_PRIMME_JDQMR_ETOL),
-    EPS_PRIMME_DEFAULT_MIN_MATVECS(EPS_PRIMME_GD_OLSEN_PLUSK), EPS_PRIMME_ARNOLDI,
-    EPS_PRIMME_GD, EPS_PRIMME_GD_PLUSK, EPS_PRIMME_GD_OLSEN_PLUSK,
-    EPS_PRIMME_JD_OLSEN_PLUSK, EPS_PRIMME_RQI, EPS_PRIMME_JDQR, EPS_PRIMME_JDQMR,
-    EPS_PRIMME_JDQMR_ETOL, EPS_PRIMME_SUBSPACE_ITERATION,
-    EPS_PRIMME_LOBPCG_ORTHOBASIS, EPS_PRIMME_LOBPCG_ORTHOBASISW
+-  method - method that will be used by PRIMME
 
    Options Database Key:
-.  -eps_primme_set_method - Sets the method for the PRIMME library.
+.  -eps_primme_method - Sets the method for the PRIMME library
 
    Note:
    If not set, the method defaults to EPS_PRIMME_DEFAULT_MIN_TIME.
@@ -511,30 +463,24 @@ static PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps,EPSPRIMMEMethod *method)
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
 
   PetscFunctionBegin;
-  if (method) *method = (EPSPRIMMEMethod)ops->method;
+  *method = (EPSPRIMMEMethod)ops->method;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSPRIMMEGetMethod"
-/*@C
-    EPSPRIMMEGetMethod - Gets the method for the PRIMME library.
+/*@
+   EPSPRIMMEGetMethod - Gets the method for the PRIMME library.
 
-    Mon Collective on EPS
+   Not Collective
 
-   Input Parameters:
+   Input Parameter:
 .  eps - the eigenproblem solver context
 
-   Output Parameters:
-.  method - method that will be used by PRIMME. It must be one of:
-    EPS_PRIMME_DYNAMIC, EPS_PRIMME_DEFAULT_MIN_TIME(EPS_PRIMME_JDQMR_ETOL),
-    EPS_PRIMME_DEFAULT_MIN_MATVECS(EPS_PRIMME_GD_OLSEN_PLUSK), EPS_PRIMME_ARNOLDI,
-    EPS_PRIMME_GD, EPS_PRIMME_GD_PLUSK, EPS_PRIMME_GD_OLSEN_PLUSK,
-    EPS_PRIMME_JD_OLSEN_PLUSK, EPS_PRIMME_RQI, EPS_PRIMME_JDQR, EPS_PRIMME_JDQMR,
-    EPS_PRIMME_JDQMR_ETOL, EPS_PRIMME_SUBSPACE_ITERATION,
-    EPS_PRIMME_LOBPCG_ORTHOBASIS, EPS_PRIMME_LOBPCG_ORTHOBASISW
+   Output Parameter:
+.  method - method that will be used by PRIMME
 
-    Level: advanced
+   Level: advanced
 
 .seealso: EPSPRIMMESetMethod(), EPSPRIMMEMethod
 @*/
@@ -544,7 +490,8 @@ PetscErrorCode EPSPRIMMEGetMethod(EPS eps,EPSPRIMMEMethod *method)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = PetscTryMethod(eps,"EPSPRIMMEGetMethod_C",(EPS,EPSPRIMMEMethod*),(eps,method));CHKERRQ(ierr);
+  PetscValidPointer(method,2);
+  ierr = PetscUseMethod(eps,"EPSPRIMMEGetMethod_C",(EPS,EPSPRIMMEMethod*),(eps,method));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -556,15 +503,15 @@ PETSC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
   EPS_PRIMME     *primme;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(eps,EPS_PRIMME,&primme);CHKERRQ(ierr);
-  eps->data                      = (void*)primme;
-  eps->ops->setup                = EPSSetUp_PRIMME;
-  eps->ops->setfromoptions       = EPSSetFromOptions_PRIMME;
-  eps->ops->destroy              = EPSDestroy_PRIMME;
-  eps->ops->reset                = EPSReset_PRIMME;
-  eps->ops->view                 = EPSView_PRIMME;
-  eps->ops->backtransform        = EPSBackTransform_Default;
-  eps->ops->computevectors       = EPSComputeVectors_Default;
+  ierr = PetscNewLog(eps,&primme);CHKERRQ(ierr);
+  eps->data = (void*)primme;
+
+  eps->ops->setup          = EPSSetUp_PRIMME;
+  eps->ops->setfromoptions = EPSSetFromOptions_PRIMME;
+  eps->ops->destroy        = EPSDestroy_PRIMME;
+  eps->ops->reset          = EPSReset_PRIMME;
+  eps->ops->view           = EPSView_PRIMME;
+  eps->ops->backtransform  = EPSBackTransform_Default;
 
   primme_initialize(&primme->primme);
   primme->primme.matrixMatvec = multMatvec_PRIMME;
