@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -22,8 +22,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/epsimpl.h>     /*I "slepceps.h" I*/
-#include <slepcblaslapack.h>
+#include <slepc/private/epsimpl.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_LAPACK"
@@ -31,7 +30,7 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 {
   PetscErrorCode ierr,ierra,ierrb;
   PetscBool      isshift,denseok=PETSC_FALSE;
-  Mat            A,B,OP,Adense,Bdense;
+  Mat            A,B,OP,Adense=NULL,Bdense=NULL;
   PetscScalar    shift,*Ap,*Bp;
   PetscInt       i,ld,nmat;
   KSP            ksp;
@@ -43,8 +42,9 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->which) { ierr = EPSSetWhichEigenpairs_Default(eps);CHKERRQ(ierr); }
   if (eps->balance!=EPS_BALANCE_NONE) { ierr = PetscInfo(eps,"Warning: balancing ignored\n");CHKERRQ(ierr); }
+  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"User-defined stopping test not supported");
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
-  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
+  ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
 
   /* attempt to get dense representations of A and B separately */
   ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&isshift);CHKERRQ(ierr);
@@ -54,14 +54,14 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
     PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
     ierra = SlepcMatConvertSeqDense(A,&Adense);CHKERRQ(ierr);
-    if (eps->isgeneralized) {
+    if (nmat>1) {
       ierrb = SlepcMatConvertSeqDense(B,&Bdense);CHKERRQ(ierr);
     } else {
       ierrb = 0;
     }
     PetscPopErrorHandler();
-    denseok = (ierra == 0 && ierrb == 0)? PETSC_TRUE: PETSC_FALSE;
-  } else Adense = NULL;
+    denseok = PetscNot(ierra || ierrb);
+  }
 
   /* setup DS */
   if (denseok) {
@@ -137,7 +137,8 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       n=eps->n,i,low,high;
-  PetscScalar    *array,*pX,*pY;
+  PetscScalar    *array,*pX;
+  Vec            v;
 
   PetscFunctionBegin;
   ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
@@ -147,42 +148,21 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
   ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
   ierr = DSGetArray(eps->ds,DS_MAT_X,&pX);CHKERRQ(ierr);
   for (i=0;i<eps->ncv;i++) {
-    ierr = VecGetOwnershipRange(eps->V[i],&low,&high);CHKERRQ(ierr);
-    ierr = VecGetArray(eps->V[i],&array);CHKERRQ(ierr);
+    ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
+    ierr = VecGetOwnershipRange(v,&low,&high);CHKERRQ(ierr);
+    ierr = VecGetArray(v,&array);CHKERRQ(ierr);
     ierr = PetscMemcpy(array,pX+i*n+low,(high-low)*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = VecRestoreArray(eps->V[i],&array);CHKERRQ(ierr);
+    ierr = VecRestoreArray(v,&array);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
   }
   ierr = DSRestoreArray(eps->ds,DS_MAT_X,&pX);CHKERRQ(ierr);
 
-  /* left eigenvectors */
-  if (eps->leftvecs) {
-    ierr = DSVectors(eps->ds,DS_MAT_Y,NULL,NULL);CHKERRQ(ierr);
-    ierr = DSGetArray(eps->ds,DS_MAT_Y,&pY);CHKERRQ(ierr);
-    for (i=0;i<eps->ncv;i++) {
-      ierr = VecGetOwnershipRange(eps->W[i],&low,&high);CHKERRQ(ierr);
-      ierr = VecGetArray(eps->W[i],&array);CHKERRQ(ierr);
-      ierr = PetscMemcpy(array,pY+i*n+low,(high-low)*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(eps->W[i],&array);CHKERRQ(ierr);
-    }
-    ierr = DSRestoreArray(eps->ds,DS_MAT_Y,&pY);CHKERRQ(ierr);
-  }
   eps->nconv  = eps->ncv;
   eps->its    = 1;
   eps->reason = EPS_CONVERGED_TOL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_LAPACK"
-PetscErrorCode EPSReset_LAPACK(EPS eps)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__
 #define __FUNCT__ "EPSCreate_LAPACK"
 PETSC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
@@ -190,9 +170,7 @@ PETSC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
   PetscFunctionBegin;
   eps->ops->solve                = EPSSolve_LAPACK;
   eps->ops->setup                = EPSSetUp_LAPACK;
-  eps->ops->reset                = EPSReset_LAPACK;
   eps->ops->backtransform        = EPSBackTransform_Default;
-  eps->ops->computevectors       = EPSComputeVectors_Default;
   PetscFunctionReturn(0);
 }
 
