@@ -161,12 +161,12 @@ static PetscErrorCode NEPNLEIGSLejaBagbyPoints(NEP nep)
       nrs[i] *= ((ds[i]-s[k-1])/(1.0-ds[i]/xi[k-1]))/beta[k-1];
       if (PetscAbsScalar(nrs[i])>maxnrs) {maxnrs = PetscAbsScalar(nrs[i]); s[k] = ds[i];}
     }
-    if (ndptx>=k) {
+    if (ndptx>k) {
       for (i=1;i<ndptx;i++) {
         nrxi[i] *= ((dxi[i]-s[k-1])/(1.0-dxi[i]/xi[k-1]))/beta[k-1];
         if (PetscAbsScalar(nrxi[i])<minnrxi) {minnrxi = PetscAbsScalar(nrxi[i]); xi[k] = dxi[i];}
       }
-    }  else xi[k] = PETSC_INFINITY;
+    } else xi[k] = PETSC_INFINITY;
     beta[k] = maxnrs;
   }
   ierr = PetscFree5(ds,dsi,dxi,nrs,nrxi);CHKERRQ(ierr);
@@ -462,7 +462,7 @@ static PetscErrorCode NEPNLEIGSDividedDifferences_split(NEP nep)
   if (!ctx->ksp) { ierr = NEPNLEIGSGetKSPs(nep,&ctx->ksp);CHKERRQ(ierr); }
   ierr = PetscObjectTypeCompare((PetscObject)nep->A[0],MATSHELL,&shell);CHKERRQ(ierr);
   for (i=0;i<ctx->nshiftsw;i++) {
-    ierr = NEPNLEIGSEvalNRTFunct(nep,ctx->nmat,ctx->shifts[i],coeffs);CHKERRQ(ierr);
+    ierr = NEPNLEIGSEvalNRTFunct(nep,ctx->nmat-1,ctx->shifts[i],coeffs);CHKERRQ(ierr);
     if (!shell) {
       ierr = MatDuplicate(nep->A[0],MAT_COPY_VALUES,&T);CHKERRQ(ierr);
     } else {
@@ -557,7 +557,7 @@ static PetscErrorCode NEPNLEIGSDividedDifferences_callback(NEP nep)
   }
   if (!ctx->ksp) { ierr = NEPNLEIGSGetKSPs(nep,&ctx->ksp);CHKERRQ(ierr); }
   for (i=0;i<ctx->nshiftsw;i++) {
-    ierr = NEPNLEIGSEvalNRTFunct(nep,ctx->nmat,ctx->shifts[i],coeffs);CHKERRQ(ierr);
+    ierr = NEPNLEIGSEvalNRTFunct(nep,ctx->nmat-1,ctx->shifts[i],coeffs);CHKERRQ(ierr);
     ierr = MatDuplicate(ctx->D[0],MAT_COPY_VALUES,&T);CHKERRQ(ierr);
     if (coeffs[0]!=1.0) { ierr = MatScale(T,coeffs[0]);CHKERRQ(ierr); }
     for (j=1;j<ctx->nmat-1;j++) {
