@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -21,211 +21,16 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/nepimpl.h>      /*I "slepcnep.h" I*/
+#include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
 
 PetscFunctionList NEPList = 0;
 PetscBool         NEPRegisterAllCalled = PETSC_FALSE;
 PetscClassId      NEP_CLASSID = 0;
-PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Dense = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0;
-static PetscBool  NEPPackageInitialized = PETSC_FALSE;
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPFinalizePackage"
-/*@C
-   NEPFinalizePackage - This function destroys everything in the Slepc interface
-   to the NEP package. It is called from SlepcFinalize().
-
-   Level: developer
-
-.seealso: SlepcFinalize()
-@*/
-PetscErrorCode NEPFinalizePackage(void)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscFunctionListDestroy(&NEPList);CHKERRQ(ierr);
-  NEPPackageInitialized = PETSC_FALSE;
-  NEPRegisterAllCalled  = PETSC_FALSE;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPInitializePackage"
-/*@C
-   NEPInitializePackage - This function initializes everything in the NEP package. It is called
-   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to NEPCreate()
-   when using static libraries.
-
-   Level: developer
-
-.seealso: SlepcInitialize()
-@*/
-PetscErrorCode NEPInitializePackage(void)
-{
-  char           logList[256];
-  char           *className;
-  PetscBool      opt;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (NEPPackageInitialized) PetscFunctionReturn(0);
-  NEPPackageInitialized = PETSC_TRUE;
-  /* Register Classes */
-  ierr = PetscClassIdRegister("Nonlinear Eigenvalue Problem solver",&NEP_CLASSID);CHKERRQ(ierr);
-  /* Register Constructors */
-  ierr = NEPRegisterAll();CHKERRQ(ierr);
-  /* Register Events */
-  ierr = PetscLogEventRegister("NEPSetUp",NEP_CLASSID,&NEP_SetUp);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("NEPSolve",NEP_CLASSID,&NEP_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("NEPDense",NEP_CLASSID,&NEP_Dense);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("NEPFunctionEval",NEP_CLASSID,&NEP_FunctionEval);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("NEPJacobianEval",NEP_CLASSID,&NEP_JacobianEval);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrstr(logList,"nep",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(NEP_CLASSID);CHKERRQ(ierr);
-    }
-  }
-  /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrstr(logList,"nep",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(NEP_CLASSID);CHKERRQ(ierr);
-    }
-  }
-  ierr = PetscRegisterFinalize(NEPFinalizePackage);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPView"
-/*@C
-   NEPView - Prints the NEP data structure.
-
-   Collective on NEP
-
-   Input Parameters:
-+  nep - the nonlinear eigenproblem solver context
--  viewer - optional visualization context
-
-   Options Database Key:
-.  -nep_view -  Calls NEPView() at end of NEPSolve()
-
-   Note:
-   The available visualization contexts include
-+     PETSC_VIEWER_STDOUT_SELF - standard output (default)
--     PETSC_VIEWER_STDOUT_WORLD - synchronized standard
-         output where only the first processor opens
-         the file.  All other processors send their
-         data to the first processor to print.
-
-   The user can open an alternative visualization context with
-   PetscViewerASCIIOpen() - output to a specified file.
-
-   Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
-@*/
-PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
-{
-  PetscErrorCode ierr;
-  char           str[50];
-  PetscBool      isascii,isslp;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
-  PetscCheckSameComm(nep,1,viewer,2);
-
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  if (isascii) {
-    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)nep,viewer,"NEP Object");CHKERRQ(ierr);
-    if (nep->ops->view) {
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-      ierr = (*nep->ops->view)(nep,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-    }
-    if (nep->split) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator in split form\n");CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator from user callbacks\n");CHKERRQ(ierr);
-    }
-    ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
-    ierr = SlepcSNPrintfScalar(str,50,nep->target,PETSC_FALSE);CHKERRQ(ierr);
-    if (!nep->which) {
-      ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
-    } else switch (nep->which) {
-      case NEP_TARGET_MAGNITUDE:
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (in magnitude)\n",str);CHKERRQ(ierr);
-        break;
-      case NEP_TARGET_REAL:
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (along the real axis)\n",str);CHKERRQ(ierr);
-        break;
-      case NEP_TARGET_IMAGINARY:
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (along the imaginary axis)\n",str);CHKERRQ(ierr);
-        break;
-      case NEP_LARGEST_MAGNITUDE:
-        ierr = PetscViewerASCIIPrintf(viewer,"largest eigenvalues in magnitude\n");CHKERRQ(ierr);
-        break;
-      case NEP_SMALLEST_MAGNITUDE:
-        ierr = PetscViewerASCIIPrintf(viewer,"smallest eigenvalues in magnitude\n");CHKERRQ(ierr);
-        break;
-      case NEP_LARGEST_REAL:
-        ierr = PetscViewerASCIIPrintf(viewer,"largest real parts\n");CHKERRQ(ierr);
-        break;
-      case NEP_SMALLEST_REAL:
-        ierr = PetscViewerASCIIPrintf(viewer,"smallest real parts\n");CHKERRQ(ierr);
-        break;
-      case NEP_LARGEST_IMAGINARY:
-        ierr = PetscViewerASCIIPrintf(viewer,"largest imaginary parts\n");CHKERRQ(ierr);
-        break;
-      case NEP_SMALLEST_IMAGINARY:
-        ierr = PetscViewerASCIIPrintf(viewer,"smallest imaginary parts\n");CHKERRQ(ierr);
-        break;
-      default: SETERRQ(PetscObjectComm((PetscObject)nep),1,"Wrong value of nep->which");
-    }
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",nep->nev);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",nep->ncv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",nep->mpd);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",nep->max_it);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of function evaluations: %D\n",nep->max_funcs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  tolerances: relative=%G, absolute=%G, solution=%G\n",nep->rtol,nep->abstol,nep->stol);CHKERRQ(ierr);
-    if (nep->lag) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  updating the preconditioner every %D iterations\n",nep->lag);CHKERRQ(ierr);
-    }
-    if (nep->cctol) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  using a constant tolerance for the linear solver\n");CHKERRQ(ierr);
-    }
-    if (nep->nini) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(nep->nini));CHKERRQ(ierr);
-    }
-  } else {
-    if (nep->ops->view) {
-      ierr = (*nep->ops->view)(nep,viewer);CHKERRQ(ierr);
-    }
-  }
-  if (!nep->ip) { ierr = NEPGetIP(nep,&nep->ip);CHKERRQ(ierr); }
-  ierr = IPView(nep->ip,viewer);CHKERRQ(ierr);
-  if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
-  ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
-  ierr = DSView(nep->ds,viewer);CHKERRQ(ierr);
-  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)nep,NEPSLP,&isslp);CHKERRQ(ierr);
-  if (!isslp) {
-    if (!nep->ksp) { ierr = NEPGetKSP(nep,&nep->ksp);CHKERRQ(ierr); }
-    ierr = KSPView(nep->ksp,viewer);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
+PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_DerivativesEval = 0;
 
 #undef __FUNCT__
 #define __FUNCT__ "NEPCreate"
-/*@C
+/*@
    NEPCreate - Creates the default NEP context.
 
    Collective on MPI_Comm
@@ -248,68 +53,70 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   PetscFunctionBegin;
   PetscValidPointer(outnep,2);
   *outnep = 0;
-#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
   ierr = NEPInitializePackage();CHKERRQ(ierr);
-#endif
-
-  ierr = SlepcHeaderCreate(nep,_p_NEP,struct _NEPOps,NEP_CLASSID,"NEP","Nonlinear Eigenvalue Problem","NEP",comm,NEPDestroy,NEPView);CHKERRQ(ierr);
+  ierr = SlepcHeaderCreate(nep,NEP_CLASSID,"NEP","Nonlinear Eigenvalue Problem","NEP",comm,NEPDestroy,NEPView);CHKERRQ(ierr);
 
   nep->max_it          = 0;
-  nep->max_funcs       = 0;
   nep->nev             = 1;
   nep->ncv             = 0;
   nep->mpd             = 0;
-  nep->lag             = 1;
   nep->nini            = 0;
-  nep->allocated_ncv   = 0;
-  nep->ip              = 0;
-  nep->ds              = 0;
-  nep->function        = 0;
-  nep->function_pre    = 0;
-  nep->jacobian        = 0;
-  nep->abstol          = PETSC_DEFAULT;
-  nep->rtol            = PETSC_DEFAULT;
-  nep->stol            = PETSC_DEFAULT;
-  nep->ktol            = 0.1;
-  nep->cctol           = PETSC_FALSE;
-  nep->ttol            = 0.0;
+  nep->target          = 0.0;
+  nep->tol             = PETSC_DEFAULT;
+  nep->conv            = NEP_CONV_REL;
+  nep->stop            = NEP_STOP_BASIC;
   nep->which           = (NEPWhich)0;
+  nep->refine          = NEP_REFINE_NONE;
+  nep->npart           = 1;
+  nep->rtol            = PETSC_DEFAULT;
+  nep->rits            = PETSC_DEFAULT;
+  nep->scheme          = (NEPRefineScheme)0;
+  nep->trackall        = PETSC_FALSE;
+
   nep->computefunction = NULL;
   nep->computejacobian = NULL;
-  nep->comparison      = NULL;
-  nep->converged       = NEPConvergedDefault;
-  nep->convergeddestroy= NULL;
-  nep->comparisonctx   = NULL;
-  nep->convergedctx    = NULL;
   nep->functionctx     = NULL;
   nep->jacobianctx     = NULL;
+  nep->computederivatives = NULL;
+  nep->derivativesctx  = NULL;
+  nep->converged       = NEPConvergedRelative;
+  nep->convergeddestroy= NULL;
+  nep->stopping        = NEPStoppingBasic;
+  nep->stoppingdestroy = NULL;
+  nep->convergedctx    = NULL;
+  nep->stoppingctx     = NULL;
+  nep->numbermonitors  = 0;
+
+  nep->ds              = NULL;
   nep->V               = NULL;
-  nep->IS              = NULL;
-  nep->eig             = NULL;
-  nep->errest          = NULL;
-  nep->data            = NULL;
-  nep->t               = NULL;
-  nep->split           = PETSC_FALSE;
-  nep->nt              = 0;
-  nep->mstr            = DIFFERENT_NONZERO_PATTERN;
+  nep->rg              = NULL;
+  nep->function        = NULL;
+  nep->function_pre    = NULL;
+  nep->jacobian        = NULL;
+  nep->derivatives     = NULL;
   nep->A               = NULL;
   nep->f               = NULL;
-  nep->nconv           = 0;
-  nep->its             = 0;
+  nep->nt              = 0;
+  nep->mstr            = DIFFERENT_NONZERO_PATTERN;
+  nep->IS              = NULL;
+  nep->eigr            = NULL;
+  nep->eigi            = NULL;
+  nep->errest          = NULL;
   nep->perm            = NULL;
-  nep->nfuncs          = 0;
-  nep->linits          = 0;
   nep->nwork           = 0;
   nep->work            = NULL;
-  nep->setupcalled     = 0;
-  nep->reason          = NEP_CONVERGED_ITERATING;
-  nep->numbermonitors  = 0;
-  nep->trackall        = PETSC_FALSE;
-  nep->rand            = 0;
+  nep->data            = NULL;
 
-  ierr = PetscRandomCreate(comm,&nep->rand);CHKERRQ(ierr);
-  ierr = PetscRandomSetSeed(nep->rand,0x12345678);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(nep,nep->rand);CHKERRQ(ierr);
+  nep->state           = NEP_STATE_INITIAL;
+  nep->nconv           = 0;
+  nep->its             = 0;
+  nep->n               = 0;
+  nep->nloc            = 0;
+  nep->nrma            = NULL;
+  nep->fui             = (NEPUserInterface)0;
+  nep->reason          = NEP_CONVERGED_ITERATING;
+
+  ierr = PetscNewLog(nep,&nep->sc);CHKERRQ(ierr);
   *outnep = nep;
   PetscFunctionReturn(0);
 }
@@ -362,7 +169,7 @@ PetscErrorCode NEPSetType(NEP nep,NEPType type)
   if (nep->ops->destroy) { ierr = (*nep->ops->destroy)(nep);CHKERRQ(ierr); }
   ierr = PetscMemzero(nep->ops,sizeof(struct _NEPOps));CHKERRQ(ierr);
 
-  nep->setupcalled = 0;
+  nep->state = NEP_STATE_INITIAL;
   ierr = PetscObjectChangeTypeName((PetscObject)nep,type);CHKERRQ(ierr);
   ierr = (*r)(nep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -397,7 +204,7 @@ PetscErrorCode NEPGetType(NEP nep,NEPType *type)
 #undef __FUNCT__
 #define __FUNCT__ "NEPRegister"
 /*@C
-   NEPRegister - Adds a method to the quadratic eigenproblem solver package.
+   NEPRegister - Adds a method to the nonlinear eigenproblem solver package.
 
    Not Collective
 
@@ -410,13 +217,13 @@ PetscErrorCode NEPGetType(NEP nep,NEPType *type)
 
    Sample usage:
 .vb
-   NEPRegister("my_solver",MySolverCreate);
+    NEPRegister("my_solver",MySolverCreate);
 .ve
 
    Then, your solver can be chosen with the procedural interface via
-$     NEPSetType(qep,"my_solver")
+$     NEPSetType(nep,"my_solver")
    or at runtime via the option
-$     -qep_type my_solver
+$     -nep_type my_solver
 
    Level: advanced
 
@@ -431,10 +238,36 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "NEPReset_Problem"
+/*
+   NEPReset_Problem - Destroys the problem matrices.
+@*/
+PetscErrorCode NEPReset_Problem(NEP nep)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  ierr = MatDestroy(&nep->function);CHKERRQ(ierr);
+  ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
+  ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
+  ierr = MatDestroy(&nep->derivatives);CHKERRQ(ierr);
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
+    ierr = MatDestroyMatrices(nep->nt,&nep->A);CHKERRQ(ierr);
+    for (i=0;i<nep->nt;i++) {
+      ierr = FNDestroy(&nep->f[i]);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(nep->f);CHKERRQ(ierr);
+    ierr = PetscFree(nep->nrma);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 #undef __FUNCT__
 #define __FUNCT__ "NEPReset"
 /*@
-   NEPReset - Resets the NEP context to the setupcalled=0 state and removes any
+   NEPReset - Resets the NEP context to the initial state and removes any
    allocated objects.
 
    Collective on NEP
@@ -449,23 +282,29 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
 PetscErrorCode NEPReset(NEP nep)
 {
   PetscErrorCode ierr;
+  PetscInt       ncols;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (nep->ops->reset) { ierr = (nep->ops->reset)(nep);CHKERRQ(ierr); }
-  if (nep->ip) { ierr = IPReset(nep->ip);CHKERRQ(ierr); }
   if (nep->ds) { ierr = DSReset(nep->ds);CHKERRQ(ierr); }
-  ierr = VecDestroy(&nep->t);CHKERRQ(ierr);
-  ierr = NEPFreeSolution(nep);CHKERRQ(ierr);
-  nep->nfuncs      = 0;
-  nep->linits      = 0;
-  nep->setupcalled = 0;
+  ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
+  ierr = BVGetSizes(nep->V,NULL,NULL,&ncols);CHKERRQ(ierr);
+  if (ncols) {
+    ierr = PetscFree4(nep->eigr,nep->eigi,nep->errest,nep->perm);CHKERRQ(ierr);
+  }
+  ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(nep->nwork,&nep->work);CHKERRQ(ierr);
+  ierr = KSPDestroy(&nep->refineksp);CHKERRQ(ierr);
+  ierr = PetscSubcommDestroy(&nep->refinesubc);CHKERRQ(ierr);
+  nep->nwork = 0;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "NEPDestroy"
-/*@C
+/*@
    NEPDestroy - Destroys the NEP context.
 
    Collective on NEP
@@ -480,7 +319,6 @@ PetscErrorCode NEPReset(NEP nep)
 PetscErrorCode NEPDestroy(NEP *nep)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
 
   PetscFunctionBegin;
   if (!*nep) PetscFunctionReturn(0);
@@ -488,66 +326,58 @@ PetscErrorCode NEPDestroy(NEP *nep)
   if (--((PetscObject)(*nep))->refct > 0) { *nep = 0; PetscFunctionReturn(0); }
   ierr = NEPReset(*nep);CHKERRQ(ierr);
   if ((*nep)->ops->destroy) { ierr = (*(*nep)->ops->destroy)(*nep);CHKERRQ(ierr); }
-  ierr = KSPDestroy(&(*nep)->ksp);CHKERRQ(ierr);
-  ierr = IPDestroy(&(*nep)->ip);CHKERRQ(ierr);
+  ierr = RGDestroy(&(*nep)->rg);CHKERRQ(ierr);
   ierr = DSDestroy(&(*nep)->ds);CHKERRQ(ierr);
-  ierr = MatDestroy(&(*nep)->function);CHKERRQ(ierr);
-  ierr = MatDestroy(&(*nep)->function_pre);CHKERRQ(ierr);
-  ierr = MatDestroy(&(*nep)->jacobian);CHKERRQ(ierr);
-  if ((*nep)->split) {
-    ierr = MatDestroyMatrices((*nep)->nt,&(*nep)->A);CHKERRQ(ierr);
-    for (i=0;i<(*nep)->nt;i++) {
-      ierr = FNDestroy(&(*nep)->f[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree((*nep)->f);CHKERRQ(ierr);
-  }
-  ierr = PetscRandomDestroy(&(*nep)->rand);CHKERRQ(ierr);
+  ierr = PetscFree((*nep)->sc);CHKERRQ(ierr);
   /* just in case the initial vectors have not been used */
   ierr = SlepcBasisDestroy_Private(&(*nep)->nini,&(*nep)->IS);CHKERRQ(ierr);
+  if ((*nep)->convergeddestroy) {
+    ierr = (*(*nep)->convergeddestroy)((*nep)->convergedctx);CHKERRQ(ierr);
+  }
   ierr = NEPMonitorCancel(*nep);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(nep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "NEPSetIP"
+#define __FUNCT__ "NEPSetBV"
 /*@
-   NEPSetIP - Associates an inner product object to the nonlinear eigensolver.
+   NEPSetBV - Associates a basis vectors object to the nonlinear eigensolver.
 
    Collective on NEP
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
--  ip  - the inner product object
+-  bv  - the basis vectors object
 
    Note:
-   Use NEPGetIP() to retrieve the inner product context (for example,
+   Use NEPGetBV() to retrieve the basis vectors context (for example,
    to free it at the end of the computations).
 
    Level: advanced
 
-.seealso: NEPGetIP()
+.seealso: NEPGetBV()
 @*/
-PetscErrorCode NEPSetIP(NEP nep,IP ip)
+PetscErrorCode NEPSetBV(NEP nep,BV bv)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidHeaderSpecific(ip,IP_CLASSID,2);
-  PetscCheckSameComm(nep,1,ip,2);
-  ierr = PetscObjectReference((PetscObject)ip);CHKERRQ(ierr);
-  ierr = IPDestroy(&nep->ip);CHKERRQ(ierr);
-  nep->ip = ip;
-  ierr = PetscLogObjectParent(nep,nep->ip);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(bv,BV_CLASSID,2);
+  PetscCheckSameComm(nep,1,bv,2);
+  ierr = PetscObjectReference((PetscObject)bv);CHKERRQ(ierr);
+  ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
+  nep->V = bv;
+  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "NEPGetIP"
-/*@C
-   NEPGetIP - Obtain the inner product object associated
-   to the nonlinear eigensolver object.
+#define __FUNCT__ "NEPGetBV"
+/*@
+   NEPGetBV - Obtain the basis vectors object associated to the nonlinear
+   eigensolver object.
 
    Not Collective
 
@@ -555,24 +385,91 @@ PetscErrorCode NEPSetIP(NEP nep,IP ip)
 .  nep - eigensolver context obtained from NEPCreate()
 
    Output Parameter:
-.  ip - inner product context
+.  bv - basis vectors context
 
    Level: advanced
 
-.seealso: NEPSetIP()
+.seealso: NEPSetBV()
 @*/
-PetscErrorCode NEPGetIP(NEP nep,IP *ip)
+PetscErrorCode NEPGetBV(NEP nep,BV *bv)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(ip,2);
-  if (!nep->ip) {
-    ierr = IPCreate(PetscObjectComm((PetscObject)nep),&nep->ip);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent(nep,nep->ip);CHKERRQ(ierr);
+  PetscValidPointer(bv,2);
+  if (!nep->V) {
+    ierr = BVCreate(PetscObjectComm((PetscObject)nep),&nep->V);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V);CHKERRQ(ierr);
   }
-  *ip = nep->ip;
+  *bv = nep->V;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "NEPSetRG"
+/*@
+   NEPSetRG - Associates a region object to the nonlinear eigensolver.
+
+   Collective on NEP
+
+   Input Parameters:
++  nep - eigensolver context obtained from NEPCreate()
+-  rg  - the region object
+
+   Note:
+   Use NEPGetRG() to retrieve the region context (for example,
+   to free it at the end of the computations).
+
+   Level: advanced
+
+.seealso: NEPGetRG()
+@*/
+PetscErrorCode NEPSetRG(NEP nep,RG rg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidHeaderSpecific(rg,RG_CLASSID,2);
+  PetscCheckSameComm(nep,1,rg,2);
+  ierr = PetscObjectReference((PetscObject)rg);CHKERRQ(ierr);
+  ierr = RGDestroy(&nep->rg);CHKERRQ(ierr);
+  nep->rg = rg;
+  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "NEPGetRG"
+/*@
+   NEPGetRG - Obtain the region object associated to the
+   nonlinear eigensolver object.
+
+   Not Collective
+
+   Input Parameters:
+.  nep - eigensolver context obtained from NEPCreate()
+
+   Output Parameter:
+.  rg - region context
+
+   Level: advanced
+
+.seealso: NEPSetRG()
+@*/
+PetscErrorCode NEPGetRG(NEP nep,RG *rg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidPointer(rg,2);
+  if (!nep->rg) {
+    ierr = RGCreate(PetscObjectComm((PetscObject)nep),&nep->rg);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg);CHKERRQ(ierr);
+  }
+  *rg = nep->rg;
   PetscFunctionReturn(0);
 }
 
@@ -606,13 +503,13 @@ PetscErrorCode NEPSetDS(NEP nep,DS ds)
   ierr = PetscObjectReference((PetscObject)ds);CHKERRQ(ierr);
   ierr = DSDestroy(&nep->ds);CHKERRQ(ierr);
   nep->ds = ds;
-  ierr = PetscLogObjectParent(nep,nep->ds);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "NEPGetDS"
-/*@C
+/*@
    NEPGetDS - Obtain the direct solver object associated to the
    nonlinear eigensolver object.
 
@@ -637,51 +534,17 @@ PetscErrorCode NEPGetDS(NEP nep,DS *ds)
   PetscValidPointer(ds,2);
   if (!nep->ds) {
     ierr = DSCreate(PetscObjectComm((PetscObject)nep),&nep->ds);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent(nep,nep->ds);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds);CHKERRQ(ierr);
   }
   *ds = nep->ds;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "NEPSetKSP"
+#define __FUNCT__ "NEPRefineGetKSP"
 /*@
-   NEPSetKSP - Associates a linear solver object to the nonlinear eigensolver.
-
-   Collective on NEP
-
-   Input Parameters:
-+  nep - eigensolver context obtained from NEPCreate()
--  ksp - the linear solver object
-
-   Note:
-   Use NEPGetKSP() to retrieve the linear solver context (for example,
-   to free it at the end of the computations).
-
-   Level: developer
-
-.seealso: NEPGetKSP()
-@*/
-PetscErrorCode NEPSetKSP(NEP nep,KSP ksp)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidHeaderSpecific(ksp,KSP_CLASSID,2);
-  PetscCheckSameComm(nep,1,ksp,2);
-  ierr = PetscObjectReference((PetscObject)ksp);CHKERRQ(ierr);
-  ierr = KSPDestroy(&nep->ksp);CHKERRQ(ierr);
-  nep->ksp = ksp;
-  ierr = PetscLogObjectParent(nep,nep->ksp);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetKSP"
-/*@C
-   NEPGetKSP - Obtain the linear solver (KSP) object associated
-   to the eigensolver object.
+   NEPRefineGetKSP - Obtain the ksp object used by the eigensolver
+   object in the refinement phase.
 
    Not Collective
 
@@ -689,27 +552,34 @@ PetscErrorCode NEPSetKSP(NEP nep,KSP ksp)
 .  nep - eigensolver context obtained from NEPCreate()
 
    Output Parameter:
-.  ksp - linear solver context
+.  ksp - ksp context
 
-   Level: beginner
+   Level: advanced
 
-.seealso: NEPSetKSP()
+.seealso: NEPSetRefine()
 @*/
-PetscErrorCode NEPGetKSP(NEP nep,KSP *ksp)
+PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(ksp,2);
-  if (!nep->ksp) {
-    ierr = KSPCreate(PetscObjectComm((PetscObject)nep),&nep->ksp);CHKERRQ(ierr);
-    ierr = KSPSetOptionsPrefix(nep->ksp,((PetscObject)nep)->prefix);CHKERRQ(ierr);
-    ierr = KSPAppendOptionsPrefix(nep->ksp,"nep_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->ksp,(PetscObject)nep,1);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent(nep,nep->ksp);CHKERRQ(ierr);
+  if (!nep->refineksp) {
+    if (nep->npart>1) {
+      /* Split in subcomunicators */
+      ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&nep->refinesubc);CHKERRQ(ierr);
+      ierr = PetscSubcommSetNumber(nep->refinesubc,nep->npart);CHKERRQ(ierr);CHKERRQ(ierr);
+      ierr = PetscSubcommSetType(nep->refinesubc,PETSC_SUBCOMM_CONTIGUOUS);CHKERRQ(ierr);
+      ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
+    }
+    ierr = KSPCreate((nep->npart==1)?PetscObjectComm((PetscObject)nep):PetscSubcommChild(nep->refinesubc),&nep->refineksp);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp);CHKERRQ(ierr);
+    ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix);CHKERRQ(ierr);
+    ierr = KSPAppendOptionsPrefix(*ksp,"nep_refine_");CHKERRQ(ierr);
+    ierr = KSPSetErrorIfNotConverged(*ksp,PETSC_TRUE);CHKERRQ(ierr);
   }
-  *ksp = nep->ksp;
+  *ksp = nep->refineksp;
   PetscFunctionReturn(0);
 }
 
@@ -724,11 +594,18 @@ PetscErrorCode NEPGetKSP(NEP nep,KSP *ksp)
 +  nep    - eigensolver context
 -  target - the value of the target
 
+   Options Database Key:
+.  -nep_target <scalar> - the value of the target
+
    Notes:
    The target is a scalar value used to determine the portion of the spectrum
    of interest. It is used in combination with NEPSetWhichEigenpairs().
 
-   Level: beginner
+   In the case of complex scalars, a complex value can be provided in the
+   command line with [+/-][realnumber][+/-]realnumberi with no spaces, e.g.
+   -nep_target 1.0+2.0i
+
+   Level: intermediate
 
 .seealso: NEPGetTarget(), NEPSetWhichEigenpairs()
 @*/
@@ -754,11 +631,11 @@ PetscErrorCode NEPSetTarget(NEP nep,PetscScalar target)
    Output Parameter:
 .  target - the value of the target
 
-   Level: beginner
-
    Note:
    If the target was not set by the user, then zero is returned.
 
+   Level: intermediate
+
 .seealso: NEPSetTarget()
 @*/
 PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
@@ -788,18 +665,20 @@ PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
          evaluation routine (may be NULL) (if NULL then NEP retains any
          previously set value)
 
-   Notes:
-   The routine fun() takes Mat* as the matrix arguments rather than Mat.
-   This allows the Function evaluation routine to replace A and/or B with a
-   completely new matrix structure (not just different matrix elements)
-   when appropriate, for instance, if the nonzero structure is changing
-   throughout the global iterations.
+   Calling Sequence of fun:
+$   fun(NEP nep,PetscScalar lambda,Mat F,Mat P,void *ctx)
+
++  nep    - the NEP context
+.  lambda - the scalar argument where T(.) must be evaluated
+.  T      - matrix that will contain T(lambda)
+.  P      - (optional) different matrix to build the preconditioner
+-  ctx    - (optional) user-defined context, as set by NEPSetFunction()
 
    Level: beginner
 
 .seealso: NEPGetFunction(), NEPSetJacobian()
 @*/
-PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,PetscScalar,Mat*,Mat*,MatStructure*,void*),void *ctx)
+PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,PetscScalar,Mat,Mat,void*),void *ctx)
 {
   PetscErrorCode ierr;
 
@@ -809,6 +688,11 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
   if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,3);
   if (A) PetscCheckSameComm(nep,1,A,2);
   if (B) PetscCheckSameComm(nep,1,B,3);
+
+  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) {  /* clean previous user info */
+    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
+  }
+
   if (fun) nep->computefunction = fun;
   if (ctx) nep->functionctx     = ctx;
   if (A) {
@@ -821,7 +705,7 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
     ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
     nep->function_pre = B;
   }
-  nep->split = PETSC_FALSE;
+  nep->fui = NEP_USER_INTERFACE_CALLBACK;
   PetscFunctionReturn(0);
 }
 
@@ -846,10 +730,11 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
 
 .seealso: NEPSetFunction()
 @*/
-PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,PetscScalar,Mat*,Mat*,MatStructure*,void*),void **ctx)
+PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,PetscScalar,Mat,Mat,void*),void **ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  NEPCheckCallback(nep,1);
   if (A)   *A   = nep->function;
   if (B)   *B   = nep->function_pre;
   if (fun) *fun = nep->computefunction;
@@ -874,18 +759,19 @@ PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,P
          evaluation routine (may be NULL) (if NULL then NEP retains any
          previously set value)
 
-   Notes:
-   The routine jac() takes Mat* as the matrix arguments rather than Mat.
-   This allows the Jacobian evaluation routine to replace A with a
-   completely new matrix structure (not just different matrix elements)
-   when appropriate, for instance, if the nonzero structure is changing
-   throughout the global iterations.
+   Calling Sequence of jac:
+$   jac(NEP nep,PetscScalar lambda,Mat J,void *ctx)
+
++  nep    - the NEP context
+.  lambda - the scalar argument where T'(.) must be evaluated
+.  J      - matrix that will contain T'(lambda)
+-  ctx    - (optional) user-defined context, as set by NEPSetJacobian()
 
    Level: beginner
 
 .seealso: NEPSetFunction(), NEPGetJacobian()
 @*/
-PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScalar,Mat*,MatStructure*,void*),void *ctx)
+PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScalar,Mat,void*),void *ctx)
 {
   PetscErrorCode ierr;
 
@@ -893,6 +779,11 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   if (A) PetscCheckSameComm(nep,1,A,2);
+
+  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) {  /* clean previous user info */
+    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
+  }
+
   if (jac) nep->computejacobian = jac;
   if (ctx) nep->jacobianctx     = ctx;
   if (A) {
@@ -900,7 +791,7 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
     ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
     nep->jacobian = A;
   }
-  nep->split = PETSC_FALSE;
+  nep->fui = NEP_USER_INTERFACE_CALLBACK;
   PetscFunctionReturn(0);
 }
 
@@ -908,7 +799,7 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
 #define __FUNCT__ "NEPGetJacobian"
 /*@C
    NEPGetJacobian - Returns the Jacobian matrix and optionally the user
-   provided context for evaluating the Jacobian.
+   provided routine and context for evaluating the Jacobian.
 
    Not Collective, but Mat object will be parallel if NEP object is
 
@@ -924,10 +815,11 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
 
 .seealso: NEPSetJacobian()
 @*/
-PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscScalar,Mat*,MatStructure*,void*),void **ctx)
+PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscScalar,Mat,void*),void **ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  NEPCheckCallback(nep,1);
   if (A)   *A   = nep->jacobian;
   if (jac) *jac = nep->computejacobian;
   if (ctx) *ctx = nep->jacobianctx;
@@ -963,7 +855,7 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
    This function must be called before NEPSetUp(). If it is called again
    after NEPSetUp() then the NEP object is reset.
 
-   Level: intermediate
+   Level: beginner
 
 .seealso: NEPGetSplitOperatorTerm(), NEPGetSplitOperatorInfo()
  @*/
@@ -980,36 +872,29 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructur
   PetscCheckSameComm(nep,1,*A,3);
   PetscValidPointer(f,4);
   PetscCheckSameComm(nep,1,*f,4);
-  if (nep->setupcalled) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
   /* clean previously stored information */
-  ierr = MatDestroy(&nep->function);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
-  if (nep->split) {
-    ierr = MatDestroyMatrices(nep->nt,&nep->A);CHKERRQ(ierr);
-    for (i=0;i<nep->nt;i++) {
-      ierr = FNDestroy(&nep->f[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(nep->f);CHKERRQ(ierr);
-  }
+  ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
   /* allocate space and copy matrices and functions */
-  ierr = PetscMalloc(n*sizeof(Mat),&nep->A);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(nep,n*sizeof(Mat));CHKERRQ(ierr);
+  ierr = PetscMalloc1(n,&nep->A);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(Mat));CHKERRQ(ierr);
   for (i=0;i<n;i++) {
     PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
     ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
     nep->A[i] = A[i];
   }
-  ierr = PetscMalloc(n*sizeof(FN),&nep->f);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(nep,n*sizeof(FN));CHKERRQ(ierr);
+  ierr = PetscMalloc1(n,&nep->f);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(FN));CHKERRQ(ierr);
   for (i=0;i<n;i++) {
     PetscValidHeaderSpecific(f[i],FN_CLASSID,4);
     ierr = PetscObjectReference((PetscObject)f[i]);CHKERRQ(ierr);
     nep->f[i] = f[i];
   }
-  nep->nt    = n;
-  nep->mstr  = str;
-  nep->split = PETSC_TRUE;
+  ierr = PetscCalloc1(n,&nep->nrma);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(PetscReal));CHKERRQ(ierr);
+  nep->nt   = n;
+  nep->mstr = str;
+  nep->fui  = NEP_USER_INTERFACE_SPLIT;
   PetscFunctionReturn(0);
 }
 
@@ -1037,7 +922,8 @@ PetscErrorCode NEPGetSplitOperatorTerm(NEP nep,PetscInt k,Mat *A,FN *f)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (k<0 || k>=nep->nt) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %d",nep->nt-1);
+  NEPCheckSplit(nep,1);
+  if (k<0 || k>=nep->nt) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",nep->nt-1);
   if (A) *A = nep->A[k];
   if (f) *f = nep->f[k];
   PetscFunctionReturn(0);
@@ -1066,8 +952,85 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (n) *n = nep->nt;
+  NEPCheckSplit(nep,1);
+  if (n)   *n = nep->nt;
   if (str) *str = nep->mstr;
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "NEPSetDerivatives"
+/*@C
+   NEPSetDerivatives - Sets the function to compute the k-th derivative T^(k)(lambda)
+   for any value of k (including 0), as well as the location to store the matrix.
+
+   Logically Collective on NEP and Mat
+
+   Input Parameters:
++  nep - the NEP context
+.  A   - the matrix to store the computed derivative
+.  der - routing to evaluate the k-th derivative (if NULL then NEP retains any
+         previously set value)
+-  ctx - [optional] user-defined context for private data for the derivatives
+         evaluation routine (may be NULL) (if NULL then NEP retains any
+         previously set value)
+
+   Level: beginner
+
+.seealso: NEPSetFunction(), NEPGetDerivatives()
+@*/
+PetscErrorCode NEPSetDerivatives(NEP nep,Mat A,PetscErrorCode (*der)(NEP,PetscScalar,PetscInt,Mat,void*),void *ctx)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  if (A) PetscCheckSameComm(nep,1,A,2);
+
+  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_DERIVATIVES) {  /* clean previous user info */
+    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
+  }
+
+  if (der) nep->computederivatives = der;
+  if (ctx) nep->derivativesctx     = ctx;
+  if (A) {
+    ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
+    ierr = MatDestroy(&nep->derivatives);CHKERRQ(ierr);
+    nep->derivatives = A;
+  }
+  nep->fui = NEP_USER_INTERFACE_DERIVATIVES;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "NEPGetDerivatives"
+/*@C
+   NEPGetDerivatives - Returns the derivatives matrix and optionally the user
+   provided routine and context for evaluating the derivatives.
+
+   Not Collective, but Mat object will be parallel if NEP object is
+
+   Input Parameter:
+.  nep - the nonlinear eigensolver context
+
+   Output Parameters:
++  A   - location to stash the derivatives matrix (or NULL)
+.  der - location to put derivatives function (or NULL)
+-  ctx - location to stash derivatives context (or NULL)
+
+   Level: advanced
+
+.seealso: NEPSetDerivatives()
+@*/
+PetscErrorCode NEPGetDerivatives(NEP nep,Mat *A,PetscErrorCode (**der)(NEP,PetscScalar,PetscInt,Mat,void*),void **ctx)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  NEPCheckDerivatives(nep,1);
+  if (A)   *A   = nep->derivatives;
+  if (der) *der = nep->computederivatives;
+  if (ctx) *ctx = nep->derivativesctx;
+  PetscFunctionReturn(0);
+}
+
