@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -21,9 +21,30 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/nepimpl.h>       /*I "slepcnep.h" I*/
+#include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
 #include <petscdraw.h>
 
+#undef __FUNCT__
+#define __FUNCT__ "NEPComputeVectors"
+PetscErrorCode NEPComputeVectors(NEP nep)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  NEPCheckSolved(nep,1);
+  switch (nep->state) {
+  case NEP_STATE_SOLVED:
+    if (nep->ops->computevectors) {
+      ierr = (*nep->ops->computevectors)(nep);CHKERRQ(ierr);
+    }
+    break;
+  default:
+    break;
+  }
+  nep->state = NEP_STATE_EIGENVECTORS;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__
 #define __FUNCT__ "NEPSolve"
 /*@
@@ -36,7 +57,11 @@
 
    Options Database Keys:
 +  -nep_view - print information about the solver used
--  -nep_plot_eigs - plot computed eigenvalues
+.  -nep_view_vectors binary - save the computed eigenvectors to the default binary viewer
+.  -nep_view_values - print computed eigenvalues
+.  -nep_converged_reason - print reason for convergence, and number of iterations
+.  -nep_error_absolute - print absolute errors of each eigenpair
+-  -nep_error_relative - print relative errors of each eigenpair
 
    Level: beginner
 
@@ -44,17 +69,12 @@
 @*/
 PetscErrorCode NEPSolve(NEP nep)
 {
-  PetscErrorCode    ierr;
-  PetscInt          i;
-  PetscReal         re,im;
-  PetscBool         flg;
-  PetscViewer       viewer;
-  PetscViewerFormat format;
-  PetscDraw         draw;
-  PetscDrawSP       drawsp;
+  PetscErrorCode ierr;
+  PetscInt       i;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  if (nep->state>=NEP_STATE_SOLVED) PetscFunctionReturn(0);
   ierr = PetscLogEventBegin(NEP_Solve,nep,0,0,0);CHKERRQ(ierr);
 
   /* call setup */
@@ -62,68 +82,40 @@ PetscErrorCode NEPSolve(NEP nep)
   nep->nconv = 0;
   nep->its = 0;
   for (i=0;i<nep->ncv;i++) {
-    nep->eig[i]   = 0.0;
+    nep->eigr[i]   = 0.0;
+    nep->eigi[i]   = 0.0;
     nep->errest[i] = 0.0;
+    nep->perm[i]   = i;
   }
-  nep->ktol = 0.1;
-  ierr = NEPMonitor(nep,nep->its,nep->nconv,nep->eig,nep->errest,nep->ncv);CHKERRQ(ierr);
-
-  ierr = DSSetEigenvalueComparison(nep->ds,nep->comparison,nep->comparisonctx);CHKERRQ(ierr);
+  ierr = NEPViewFromOptions(nep,NULL,"-nep_view_pre");CHKERRQ(ierr);
 
   ierr = (*nep->ops->solve)(nep);CHKERRQ(ierr);
+  nep->state = NEP_STATE_SOLVED;
 
   if (!nep->reason) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
 
-  /* sort eigenvalues according to nep->which parameter */
-  ierr = NEPSortEigenvalues(nep,nep->nconv,nep->eig,nep->perm);CHKERRQ(ierr);
+  if (nep->refine==NEP_REFINE_SIMPLE && nep->rits>0 && nep->nconv>0) {
+    ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
+    ierr = NEPNewtonRefinementSimple(nep,&nep->rits,nep->rtol,nep->nconv);CHKERRQ(ierr);
+    nep->state = NEP_STATE_EIGENVECTORS;
+  }
 
+  /* sort eigenvalues according to nep->which parameter */
+  ierr = SlepcSortEigenvalues(nep->sc,nep->nconv,nep->eigr,nep->eigi,nep->perm);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(NEP_Solve,nep,0,0,0);CHKERRQ(ierr);
 
   /* various viewers */
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_view",&viewer,&format,&flg);CHKERRQ(ierr);
-  if (flg && !PetscPreLoadingOn) {
-    ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
-    ierr = NEPView(nep,viewer);CHKERRQ(ierr);
-    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
-  }
-
-  flg = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(((PetscObject)nep)->prefix,"-nep_plot_eigs",&flg,NULL);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
-    ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-    ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
-    for (i=0;i<nep->nconv;i++) {
-      re = PetscRealPart(nep->eig[i]);
-      im = PetscImaginaryPart(nep->eig[i]);
-      ierr = PetscDrawSPAddPoint(drawsp,&re,&im);CHKERRQ(ierr);
-    }
-    ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
-    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
-  }
+  ierr = NEPViewFromOptions(nep,NULL,"-nep_view");CHKERRQ(ierr);
+  ierr = NEPReasonViewFromOptions(nep);CHKERRQ(ierr);
+  ierr = NEPErrorViewFromOptions(nep);CHKERRQ(ierr);
+  ierr = NEPValuesViewFromOptions(nep);CHKERRQ(ierr);
+  ierr = NEPVectorsViewFromOptions(nep);CHKERRQ(ierr);
 
   /* Remove the initial subspace */
   nep->nini = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEP_KSPSolve"
-PetscErrorCode NEP_KSPSolve(NEP nep,Vec b,Vec x)
-{
-  PetscErrorCode ierr;
-  PetscInt       lits;
-
-  PetscFunctionBegin;
-  ierr = KSPSolve(nep->ksp,b,x);CHKERRQ(ierr);
-  ierr = KSPGetIterationNumber(nep->ksp,&lits);CHKERRQ(ierr);
-  nep->linits += lits;
-  ierr = PetscInfo2(nep,"iter=%D, linear solve iterations=%D\n",nep->its,lits);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__
 #define __FUNCT__ "NEPProjectOperator"
 /*@
@@ -134,8 +126,7 @@ PetscErrorCode NEP_KSPSolve(NEP nep,Vec b,Vec x)
    Input Parameters:
 +  nep - the nonlinear eigensolver context
 .  j0  - initial index
-.  j1  - final index
--  f   - workspace vector
+-  j1  - final index
 
    Notes:
    This is available for split operator only.
@@ -151,42 +142,23 @@ PetscErrorCode NEP_KSPSolve(NEP nep,Vec b,Vec x)
 
 .seealso: NEPSetSplitOperator()
 @*/
-PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1,Vec f)
+PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1)
 {
   PetscErrorCode ierr;
-  PetscInt       i,j,k,ld;
-  PetscScalar    *G,val;
-  Vec            *V = nep->V;
-  PetscBool      isherm,set,flg;
+  PetscInt       k;
+  Mat            G;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,j0,2);
   PetscValidLogicalCollectiveInt(nep,j1,3);
-  PetscValidHeaderSpecific(f,VEC_CLASSID,4);
-  if (!nep->split) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONGSTATE,"This solver requires a split operator");
-  ierr = DSGetLeadingDimension(nep->ds,&ld);CHKERRQ(ierr);
+  NEPCheckProblem(nep,1);
+  NEPCheckSplit(nep,1);
+  ierr = BVSetActiveColumns(nep->V,j0,j1);CHKERRQ(ierr);
   for (k=0;k<nep->nt;k++) {
-    ierr = DSGetArray(nep->ds,DSMatExtra[k],&G);CHKERRQ(ierr);
-    ierr = MatIsHermitianKnown(nep->A[k],&set,&flg);CHKERRQ(ierr);
-    isherm = set? flg: PETSC_FALSE;
-    for (j=j0;j<j1;j++) {
-      if (!isherm) {
-        if (j>0) { ierr = MatMultHermitianTranspose(nep->A[k],V[j],f);CHKERRQ(ierr); }
-        ierr = VecMDot(f,j,V,G+j*ld);CHKERRQ(ierr);
-        for (i=0;i<j;i++)
-          G[j+i*ld] = PetscConj(G[i+j*ld]);
-      }
-      ierr = MatMult(nep->A[k],V[j],f);CHKERRQ(ierr);
-      ierr = VecDot(f,V[j],&val);CHKERRQ(ierr);
-      G[j+j*ld] = val;
-      ierr = VecMDot(f,j,V,G+j*ld);CHKERRQ(ierr);
-      if (isherm) {
-        for (i=0;i<j;i++)
-          G[j+i*ld] = PetscConj(G[i+j*ld]);
-      }
-    }
-    ierr = DSRestoreArray(nep->ds,DSMatExtra[k],&G);CHKERRQ(ierr);
+    ierr = DSGetMat(nep->ds,DSMatExtra[k],&G);CHKERRQ(ierr);
+    ierr = BVMatProject(nep->V,nep->A[k],nep->V,G);CHKERRQ(ierr);
+    ierr = DSRestoreMat(nep->ds,DSMatExtra[k],&G);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -202,18 +174,17 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1,Vec f)
 +  nep    - the nonlinear eigensolver context
 .  lambda - scalar argument
 .  x      - vector to be multiplied against
--  v      - workspace vector
+-  v      - workspace vector (used only in the case of split form)
 
    Output Parameters:
 +  y   - result vector
 .  A   - Function matrix
-.  B   - optional preconditioning matrix
--  flg - flag indicating matrix structure (see MatStructure enum)
+-  B   - optional preconditioning matrix
 
    Note:
    If the nonlinear operator is represented in split form, the result 
    y = T(lambda)*x is computed without building T(lambda) explicitly. In
-   that case, parameters A, B and flg are not used. Otherwise, the matrix
+   that case, parameters A and B are not used. Otherwise, the matrix
    T(lambda) is built and the effect is the same as a call to
    NEPComputeFunction() followed by a MatMult().
 
@@ -221,7 +192,7 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1,Vec f)
 
 .seealso: NEPSetSplitOperator(), NEPComputeFunction()
 @*/
-PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat *A,Mat *B,MatStructure *flg)
+PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat A,Mat B)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -231,18 +202,21 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveScalar(nep,lambda,2);
   PetscValidHeaderSpecific(x,VEC_CLASSID,3);
-  PetscValidHeaderSpecific(y,VEC_CLASSID,4);
+  if (v) PetscValidHeaderSpecific(v,VEC_CLASSID,4);
   PetscValidHeaderSpecific(y,VEC_CLASSID,5);
-  if (nep->split) {
-    ierr = VecZeroEntries(y);CHKERRQ(ierr);
+  if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,6);
+  if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,7);
+
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
+    ierr = VecSet(y,0.0);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
       ierr = FNEvaluateFunction(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
       ierr = MatMult(nep->A[i],x,v);CHKERRQ(ierr);
       ierr = VecAXPY(y,alpha,v);CHKERRQ(ierr);
     }
   } else {
-    ierr = NEPComputeFunction(nep,lambda,A,B,flg);CHKERRQ(ierr);
-    ierr = MatMult(*A,x,y);CHKERRQ(ierr);
+    ierr = NEPComputeFunction(nep,lambda,A,B);CHKERRQ(ierr);
+    ierr = MatMult(A,x,y);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -258,17 +232,16 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
 +  nep    - the nonlinear eigensolver context
 .  lambda - scalar argument
 .  x      - vector to be multiplied against
--  v      - workspace vector
+-  v      - workspace vector (used only in the case of split form)
 
    Output Parameters:
 +  y   - result vector
-.  A   - Jacobian matrix
--  flg - flag indicating matrix structure (see MatStructure enum)
+-  A   - Jacobian matrix
 
    Note:
    If the nonlinear operator is represented in split form, the result 
    y = T'(lambda)*x is computed without building T'(lambda) explicitly. In
-   that case, parameters A and flg are not used. Otherwise, the matrix
+   that case, parameter A is not used. Otherwise, the matrix
    T'(lambda) is built and the effect is the same as a call to
    NEPComputeJacobian() followed by a MatMult().
 
@@ -276,7 +249,7 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
 
 .seealso: NEPSetSplitOperator(), NEPComputeJacobian()
 @*/
-PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat *A,MatStructure *flg)
+PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat A)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -286,18 +259,20 @@ PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveScalar(nep,lambda,2);
   PetscValidHeaderSpecific(x,VEC_CLASSID,3);
-  PetscValidHeaderSpecific(y,VEC_CLASSID,4);
+  if (v) PetscValidHeaderSpecific(v,VEC_CLASSID,4);
   PetscValidHeaderSpecific(y,VEC_CLASSID,5);
-  if (nep->split) {
-    ierr = VecZeroEntries(y);CHKERRQ(ierr);
+  if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,6);
+
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
+    ierr = VecSet(y,0.0);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
       ierr = FNEvaluateDerivative(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
       ierr = MatMult(nep->A[i],x,v);CHKERRQ(ierr);
       ierr = VecAXPY(y,alpha,v);CHKERRQ(ierr);
     }
   } else {
-    ierr = NEPComputeJacobian(nep,lambda,A,flg);CHKERRQ(ierr);
-    ierr = MatMult(*A,x,y);CHKERRQ(ierr);
+    ierr = NEPComputeJacobian(nep,lambda,A);CHKERRQ(ierr);
+    ierr = MatMult(A,x,y);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -362,13 +337,14 @@ PetscErrorCode NEPGetConverged(NEP nep,PetscInt *nconv)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidIntPointer(nconv,2);
+  NEPCheckSolved(nep,1);
   *nconv = nep->nconv;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "NEPGetConvergedReason"
-/*@C
+/*@
    NEPGetConvergedReason - Gets the reason why the NEPSolve() iteration was
    stopped.
 
@@ -380,17 +356,15 @@ PetscErrorCode NEPGetConverged(NEP nep,PetscInt *nconv)
    Output Parameter:
 .  reason - negative value indicates diverged, positive value converged
 
-   Possible values for reason:
-+  NEP_CONVERGED_FNORM_ABS - function norm satisfied absolute tolerance
-.  NEP_CONVERGED_FNORM_RELATIVE - function norm satisfied relative tolerance
-.  NEP_CONVERGED_SNORM_RELATIVE - step norm satisfied relative tolerance
-.  NEP_DIVERGED_LINEAR_SOLVE - inner linear solve failed
-.  NEP_DIVERGED_FUNCTION_COUNT - reached maximum allowed function evaluations
-.  NEP_DIVERGED_MAX_IT - required more than its to reach convergence
+   Notes:
+
+   Possible values for reason are
++  NEP_CONVERGED_TOL - converged up to tolerance
+.  NEP_CONVERGED_USER - converged due to a user-defined condition
+.  NEP_DIVERGED_ITS - required more than max_it iterations to reach convergence
 .  NEP_DIVERGED_BREAKDOWN - generic breakdown in method
--  NEP_DIVERGED_FNORM_NAN - Inf or NaN detected in function evaluation
+-  NEP_DIVERGED_LINEAR_SOLVE - inner linear solve failed
 
-   Note:
    Can only be called after the call to NEPSolve() is complete.
 
    Level: intermediate
@@ -402,6 +376,7 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(reason,2);
+  NEPCheckSolved(nep,1);
   *reason = nep->reason;
   PetscFunctionReturn(0);
 }
@@ -419,13 +394,20 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
 -  i   - index of the solution
 
    Output Parameters:
-+  eig - eigenvalue
--  V   - eigenvector
++  eigr - real part of eigenvalue
+.  eigi - imaginary part of eigenvalue
+.  Vr   - real part of eigenvector
+-  Vi   - imaginary part of eigenvector
 
    Notes:
-   If PETSc is configured with real scalars, then complex eigenpairs cannot
-   be obtained. Users should use a complex-scalar configuration. This
-   behaviour is different to other SLEPc solvers such as EPS.
+   It is allowed to pass NULL for Vr and Vi, if the eigenvector is not
+   required. Otherwise, the caller must provide valid Vec objects, i.e.,
+   they must be created by the calling program with e.g. MatCreateVecs().
+
+   If the eigenvalue is real, then eigi and Vi are set to zero. If PETSc is
+   configured with complex scalars the eigenvalue is stored
+   directly in eigr (eigi is set to zero) and the eigenvector in Vr (Vi is
+   set to zero). In both cases, the user can pass NULL in eigi and Vi.
 
    The index i should be a value between 0 and nconv-1 (see NEPGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established
@@ -435,7 +417,7 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
 
 .seealso: NEPSolve(), NEPGetConverged(), NEPSetWhichEigenpairs()
 @*/
-PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eig,Vec V)
+PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eigr,PetscScalar *eigi,Vec Vr,Vec Vi)
 {
   PetscInt       k;
   PetscErrorCode ierr;
@@ -443,15 +425,42 @@ PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eig,Vec V)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,i,2);
-  if (V) { PetscValidHeaderSpecific(V,VEC_CLASSID,4); PetscCheckSameComm(nep,1,V,4); }
-  if (!nep->eig || !nep->V) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONGSTATE,"NEPSolve must be called first");
+  if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,5); PetscCheckSameComm(nep,1,Vr,5); }
+  if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,6); PetscCheckSameComm(nep,1,Vi,6); }
+  NEPCheckSolved(nep,1);
   if (i<0 || i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
 
-  if (!nep->perm) k = i;
-  else k = nep->perm[i];
+  ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
+  k = nep->perm[i];
 
-  if (eig) *eig = nep->eig[k];
-  if (V) { ierr = VecCopy(nep->V[k],V);CHKERRQ(ierr); }
+  /* eigenvalue */
+#if defined(PETSC_USE_COMPLEX)
+  if (eigr) *eigr = nep->eigr[k];
+  if (eigi) *eigi = 0;
+#else
+  if (eigr) *eigr = nep->eigr[k];
+  if (eigi) *eigi = nep->eigi[k];
+#endif
+
+  /* eigenvector */
+#if defined(PETSC_USE_COMPLEX)
+  if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
+  if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
+#else
+  if (nep->eigi[k]>0) { /* first value of conjugate pair */
+    if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
+    if (Vi) { ierr = BVCopyVec(nep->V,k+1,Vi);CHKERRQ(ierr); }
+  } else if (nep->eigi[k]<0) { /* second value of conjugate pair */
+    if (Vr) { ierr = BVCopyVec(nep->V,k-1,Vr);CHKERRQ(ierr); }
+    if (Vi) {
+      ierr = BVCopyVec(nep->V,k,Vi);CHKERRQ(ierr);
+      ierr = VecScale(Vi,-1.0);CHKERRQ(ierr);
+    }
+  } else { /* real eigenvalue */
+    if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
+    if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
+  }
+#endif
   PetscFunctionReturn(0);
 }
 
@@ -483,10 +492,9 @@ PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(errest,3);
-  if (!nep->eig) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"NEPSolve must be called first");
+  NEPCheckSolved(nep,1);
   if (i<0 || i>=nep->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
-  if (nep->perm) i = nep->perm[i];
-  if (errest) *errest = nep->errest[i];
+  if (errest) *errest = nep->errest[nep->perm[i]];
   PetscFunctionReturn(0);
 }
 
@@ -495,247 +503,120 @@ PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
 /*
    NEPComputeResidualNorm_Private - Computes the norm of the residual vector
    associated with an eigenpair.
+
+   Input Parameters:
+     lambda - eigenvalue
+     x      - eigenvector
+     w      - array of work vectors (two vectors in split form, one vector otherwise)
 */
-PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,PetscReal *norm)
+PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,Vec *w,PetscReal *norm)
 {
   PetscErrorCode ierr;
-  Vec            u;
-  Mat            T=nep->function;
-  MatStructure   mats;
+  Vec            y,z=NULL;
 
   PetscFunctionBegin;
-  ierr = VecDuplicate(nep->V[0],&u);CHKERRQ(ierr);
-  ierr = NEPComputeFunction(nep,lambda,&T,&T,&mats);CHKERRQ(ierr);
-  ierr = MatMult(T,x,u);CHKERRQ(ierr);
-  ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);
-  ierr = VecDestroy(&u);CHKERRQ(ierr);
+  y = w[0];
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) z = w[1];
+  ierr = NEPApplyFunction(nep,lambda,x,z,y,nep->function,nep->function_pre);CHKERRQ(ierr);
+  ierr = VecNorm(y,NORM_2,norm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "NEPComputeResidualNorm"
+#define __FUNCT__ "NEPComputeError"
 /*@
-   NEPComputeResidualNorm - Computes the norm of the residual vector associated with
-   the i-th computed eigenpair.
-
-   Collective on NEP
-
-   Input Parameter:
-+  nep - the nonlinear eigensolver context
--  i   - the solution index
-
-   Output Parameter:
-.  norm - the residual norm, computed as ||T(lambda)x||_2 where lambda is the
-   eigenvalue and x is the eigenvector.
-
-   Notes:
-   The index i should be a value between 0 and nconv-1 (see NEPGetConverged()).
-   Eigenpairs are indexed according to the ordering criterion established
-   with NEPSetWhichEigenpairs().
-
-   Level: beginner
-
-.seealso: NEPSolve(), NEPGetConverged(), NEPSetWhichEigenpairs()
-@*/
-PetscErrorCode NEPComputeResidualNorm(NEP nep,PetscInt i,PetscReal *norm)
-{
-  PetscErrorCode ierr;
-  Vec            x;
-  PetscScalar    lambda;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidLogicalCollectiveInt(nep,i,2);
-  PetscValidPointer(norm,3);
-  ierr = VecDuplicate(nep->V[0],&x);CHKERRQ(ierr);
-  ierr = NEPGetEigenpair(nep,i,&lambda,x);CHKERRQ(ierr);
-  ierr = NEPComputeResidualNorm_Private(nep,lambda,x,norm);CHKERRQ(ierr);
-  ierr = VecDestroy(&x);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeRelativeError_Private"
-/*
-   NEPComputeRelativeError_Private - Computes the relative error bound
-   associated with an eigenpair.
-*/
-PetscErrorCode NEPComputeRelativeError_Private(NEP nep,PetscScalar lambda,Vec x,PetscReal *error)
-{
-  PetscErrorCode ierr;
-  PetscReal      norm,er;
-
-  PetscFunctionBegin;
-  ierr = NEPComputeResidualNorm_Private(nep,lambda,x,&norm);CHKERRQ(ierr);
-  ierr = VecNorm(x,NORM_2,&er);CHKERRQ(ierr);
-  if (PetscAbsScalar(lambda) > norm) {
-    *error = norm/(PetscAbsScalar(lambda)*er);
-  } else {
-    *error = norm/er;
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeRelativeError"
-/*@
-   NEPComputeRelativeError - Computes the relative error bound associated
+   NEPComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
 
    Collective on NEP
 
    Input Parameter:
-+  nep - the nonlinear eigensolver context
--  i   - the solution index
++  nep  - the nonlinear eigensolver context
+.  i    - the solution index
+-  type - the type of error to compute
 
    Output Parameter:
-.  error - the relative error bound, computed as ||T(lambda)x||_2/||lambda*x||_2
-   where lambda is the eigenvalue and x is the eigenvector.
-   If lambda=0 the relative error is computed as ||T(lambda)x||_2/||x||_2.
+.  error - the error
+
+   Notes:
+   The error can be computed in various ways, all of them based on the residual
+   norm computed as ||T(lambda)x||_2 where lambda is the eigenvalue and x is the
+   eigenvector.
 
    Level: beginner
 
-.seealso: NEPSolve(), NEPComputeResidualNorm(), NEPGetErrorEstimate()
+.seealso: NEPErrorType, NEPSolve(), NEPGetErrorEstimate()
 @*/
-PetscErrorCode NEPComputeRelativeError(NEP nep,PetscInt i,PetscReal *error)
+PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *error)
 {
   PetscErrorCode ierr;
-  Vec            x;
-  PetscScalar    lambda;
+  Vec            xr,xi=NULL;
+  PetscInt       j,nwork,issplit=0;
+  PetscScalar    kr,ki,s;
+  PetscReal      er,z=0.0;
+  PetscBool      flg;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,i,2);
-  PetscValidPointer(error,3);
-  ierr = VecDuplicate(nep->V[0],&x);CHKERRQ(ierr);
-  ierr = NEPGetEigenpair(nep,i,&lambda,x);CHKERRQ(ierr);
-  ierr = NEPComputeRelativeError_Private(nep,lambda,x,error);CHKERRQ(ierr);
-  ierr = VecDestroy(&x);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
+  PetscValidLogicalCollectiveEnum(nep,type,3);
+  PetscValidPointer(error,4);
+  NEPCheckSolved(nep,1);
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSortEigenvalues"
-/*@
-   NEPSortEigenvalues - Sorts a list of eigenvalues according to the criterion
-   specified via NEPSetWhichEigenpairs().
-
-   Not Collective
-
-   Input Parameters:
-+  nep - the nonlinear eigensolver context
-.  n   - number of eigenvalues in the list
--  eig - pointer to the array containing the eigenvalues
-
-   Output Parameter:
-.  perm - resulting permutation
-
-   Note:
-   The result is a list of indices in the original eigenvalue array
-   corresponding to the first nev eigenvalues sorted in the specified
-   criterion.
-
-   Level: developer
-
-.seealso: NEPSetWhichEigenpairs()
-@*/
-PetscErrorCode NEPSortEigenvalues(NEP nep,PetscInt n,PetscScalar *eig,PetscInt *perm)
-{
-  PetscErrorCode ierr;
-  PetscInt       i,j,result,tmp;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidScalarPointer(eig,3);
-  PetscValidIntPointer(perm,4);
-  for (i=0;i<n;i++) perm[i] = i;
-  /* insertion sort */
-  for (i=n-1;i>=0;i--) {
-    j = i + 1;
-    while (j<n) {
-      ierr = NEPCompareEigenvalues(nep,eig[perm[i]],eig[perm[j]],&result);CHKERRQ(ierr);
-      if (result < 0) break;
-      tmp = perm[j-1]; perm[j-1] = perm[j]; perm[j] = tmp;
-      j++;
-    }
+  /* allocate work vectors */
+#if defined(PETSC_USE_COMPLEX)
+  nwork = 2;
+#else
+  nwork = 3;
+#endif
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
+    issplit = 1;
+    nwork++;  /* need an extra work vector for NEPComputeResidualNorm_Private */
   }
-  PetscFunctionReturn(0);
-}
+  ierr = NEPSetWorkVecs(nep,nwork);CHKERRQ(ierr);
+  xr = nep->work[issplit+1];
+#if !defined(PETSC_USE_COMPLEX)
+  xi = nep->work[issplit+2];
+#endif
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCompareEigenvalues"
-/*@
-   NEPCompareEigenvalues - Compares two eigenvalues according to a certain criterion.
+  /* compute residual norms */
+  ierr = NEPGetEigenpair(nep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+  if (ki) SETERRQ(PETSC_COMM_SELF,1,"Not implemented for complex eigenvalues with real scalars");
+#endif
+  ierr = NEPComputeResidualNorm_Private(nep,kr,xr,nep->work,error);CHKERRQ(ierr);
+  ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
 
-   Not Collective
-
-   Input Parameters:
-+  nep - the nonlinear eigensolver context
-.  a   - the 1st eigenvalue
--  b   - the 2nd eigenvalue
-
-   Output Parameter:
-.  res - result of comparison
-
-   Notes:
-   Returns an integer less than, equal to, or greater than zero if the first
-   eigenvalue is considered to be respectively less than, equal to, or greater
-   than the second one.
-
-   The criterion of comparison is related to the 'which' parameter set with
-   NEPSetWhichEigenpairs().
-
-   Level: developer
-
-.seealso: NEPSortEigenvalues(), NEPSetWhichEigenpairs()
-@*/
-PetscErrorCode NEPCompareEigenvalues(NEP nep,PetscScalar a,PetscScalar b,PetscInt *result)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidIntPointer(result,4);
-  if (!nep->comparison) SETERRQ(PETSC_COMM_SELF,1,"Undefined eigenvalue comparison function");
-  ierr = (*nep->comparison)(a,0.0,b,0.0,result,nep->comparisonctx);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetOperationCounters"
-/*@
-   NEPGetOperationCounters - Gets the total number of function evaluations, dot
-   products, and linear solve iterations used by the NEP object during the last
-   NEPSolve() call.
-
-   Not Collective
-
-   Input Parameter:
-.  nep - nonlinear eigensolver context
-
-   Output Parameter:
-+  nfuncs - number of function evaluations
-.  dots   - number of dot product operations
--  lits   - number of linear iterations
-
-   Notes:
-   These counters are reset to zero at each successive call to NEPSolve().
-
-   Level: intermediate
-
-@*/
-PetscErrorCode NEPGetOperationCounters(NEP nep,PetscInt* nfuncs,PetscInt* dots,PetscInt* lits)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (nfuncs) *nfuncs = nep->nfuncs;
-  if (dots) {
-    if (!nep->ip) { ierr = NEPGetIP(nep,&nep->ip);CHKERRQ(ierr); }
-    ierr = IPGetOperationCounters(nep->ip,dots);CHKERRQ(ierr);
+  /* compute error */
+  switch (type) {
+    case NEP_ERROR_ABSOLUTE:
+      break;
+    case NEP_ERROR_RELATIVE:
+      *error /= PetscAbsScalar(kr)*er;
+      break;
+    case NEP_ERROR_BACKWARD:
+      if (nep->fui!=NEP_USER_INTERFACE_SPLIT) {
+        *error = 0.0;
+        ierr = PetscInfo(nep,"Backward error only available in split form\n");CHKERRQ(ierr);
+        break;
+      }
+      /* initialization of matrix norms */
+      if (!nep->nrma[0]) {
+        for (j=0;j<nep->nt;j++) {
+          ierr = MatHasOperation(nep->A[j],MATOP_NORM,&flg);CHKERRQ(ierr);
+          if (!flg) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
+          ierr = MatNorm(nep->A[j],NORM_INFINITY,&nep->nrma[j]);CHKERRQ(ierr);
+        }
+      }
+      for (j=0;j<nep->nt;j++) {
+        ierr = FNEvaluateFunction(nep->f[j],kr,&s);CHKERRQ(ierr);
+        z = z + nep->nrma[j]*PetscAbsScalar(s);
+      }
+      *error /= z;
+      break;
+    default:
+      SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid error type");
   }
-  if (lits) *lits = nep->linits;
   PetscFunctionReturn(0);
 }
 
@@ -753,8 +634,7 @@ PetscErrorCode NEPGetOperationCounters(NEP nep,PetscInt* nfuncs,PetscInt* dots,P
 
    Output Parameters:
 +  A   - Function matrix
-.  B   - optional preconditioning matrix
--  flg - flag indicating matrix structure (see MatStructure enum)
+-  B   - optional preconditioning matrix
 
    Notes:
    NEPComputeFunction() is typically used within nonlinear eigensolvers
@@ -765,7 +645,7 @@ PetscErrorCode NEPGetOperationCounters(NEP nep,PetscInt* nfuncs,PetscInt* dots,P
 
 .seealso: NEPSetFunction(), NEPGetFunction()
 @*/
-PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat *A,Mat *B,MatStructure *flg)
+PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat A,Mat B)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -773,31 +653,31 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat *A,Mat *B,MatSt
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(flg,5);
-
-  if (nep->split) {
-
-    ierr = MatZeroEntries(*A);CHKERRQ(ierr);
+  NEPCheckProblem(nep,1);
+  switch (nep->fui) {
+  case NEP_USER_INTERFACE_CALLBACK:
+    if (!nep->computefunction) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_USER,"Must call NEPSetFunction() first");
+    ierr = PetscLogEventBegin(NEP_FunctionEval,nep,A,B,0);CHKERRQ(ierr);
+    PetscStackPush("NEP user Function function");
+    ierr = (*nep->computefunction)(nep,lambda,A,B,nep->functionctx);CHKERRQ(ierr);
+    PetscStackPop;
+    ierr = PetscLogEventEnd(NEP_FunctionEval,nep,A,B,0);CHKERRQ(ierr);
+    break;
+  case NEP_USER_INTERFACE_SPLIT:
+    ierr = MatZeroEntries(A);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
       ierr = FNEvaluateFunction(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
-      ierr = MatAXPY(*A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
+      ierr = MatAXPY(A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
     }
-    if (*A != *B) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented");
-
-  } else {
-
-    if (!nep->computefunction) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_USER,"Must call NEPSetFunction() first");
-
-    *flg = DIFFERENT_NONZERO_PATTERN;
-    ierr = PetscLogEventBegin(NEP_FunctionEval,nep,*A,*B,0);CHKERRQ(ierr);
-
-    PetscStackPush("NEP user Function function");
-    ierr = (*nep->computefunction)(nep,lambda,A,B,flg,nep->functionctx);CHKERRQ(ierr);
+    if (A != B) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented");
+    break;
+  case NEP_USER_INTERFACE_DERIVATIVES:
+    ierr = PetscLogEventBegin(NEP_DerivativesEval,nep,A,B,0);CHKERRQ(ierr);
+    PetscStackPush("NEP user Derivatives function");
+    ierr = (*nep->computederivatives)(nep,lambda,0,A,nep->derivativesctx);CHKERRQ(ierr);
     PetscStackPop;
-
-    ierr = PetscLogEventEnd(NEP_FunctionEval,nep,*A,*B,0);CHKERRQ(ierr);
-    nep->nfuncs++;
-
+    ierr = PetscLogEventEnd(NEP_DerivativesEval,nep,A,B,0);CHKERRQ(ierr);
+    break;
   }
   PetscFunctionReturn(0);
 }
@@ -815,8 +695,7 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat *A,Mat *B,MatSt
 -  lambda - the scalar argument
 
    Output Parameters:
-+  A   - Jacobian matrix
--  flg - flag indicating matrix structure (see MatStructure enum)
+.  A   - Jacobian matrix
 
    Notes:
    Most users should not need to explicitly call this routine, as it
@@ -826,7 +705,7 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat *A,Mat *B,MatSt
 
 .seealso: NEPSetJacobian(), NEPGetJacobian()
 @*/
-PetscErrorCode NEPComputeJacobian(NEP nep,PetscScalar lambda,Mat *A,MatStructure *flg)
+PetscErrorCode NEPComputeJacobian(NEP nep,PetscScalar lambda,Mat A)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -834,29 +713,30 @@ PetscErrorCode NEPComputeJacobian(NEP nep,PetscScalar lambda,Mat *A,MatStructure
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(flg,4);
-
-  if (nep->split) {
-
-    ierr = MatZeroEntries(*A);CHKERRQ(ierr);
+  NEPCheckProblem(nep,1);
+  switch (nep->fui) {
+  case NEP_USER_INTERFACE_CALLBACK:
+    if (!nep->computejacobian) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_USER,"Must call NEPSetJacobian() first");
+    ierr = PetscLogEventBegin(NEP_JacobianEval,nep,A,0,0);CHKERRQ(ierr);
+    PetscStackPush("NEP user Jacobian function");
+    ierr = (*nep->computejacobian)(nep,lambda,A,nep->jacobianctx);CHKERRQ(ierr);
+    PetscStackPop;
+    ierr = PetscLogEventEnd(NEP_JacobianEval,nep,A,0,0);CHKERRQ(ierr);
+    break;
+  case NEP_USER_INTERFACE_SPLIT:
+    ierr = MatZeroEntries(A);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
       ierr = FNEvaluateDerivative(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
-      ierr = MatAXPY(*A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
+      ierr = MatAXPY(A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
     }
-
-  } else {
-
-    if (!nep->computejacobian) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_USER,"Must call NEPSetJacobian() first");
-
-    *flg = DIFFERENT_NONZERO_PATTERN;
-    ierr = PetscLogEventBegin(NEP_JacobianEval,nep,*A,0,0);CHKERRQ(ierr);
-
-    PetscStackPush("NEP user Jacobian function");
-    ierr = (*nep->computejacobian)(nep,lambda,A,flg,nep->jacobianctx);CHKERRQ(ierr);
+    break;
+  case NEP_USER_INTERFACE_DERIVATIVES:
+    ierr = PetscLogEventBegin(NEP_DerivativesEval,nep,A,0,0);CHKERRQ(ierr);
+    PetscStackPush("NEP user Derivatives function");
+    ierr = (*nep->computederivatives)(nep,lambda,1,A,nep->derivativesctx);CHKERRQ(ierr);
     PetscStackPop;
-
-    ierr = PetscLogEventEnd(NEP_JacobianEval,nep,*A,0,0);CHKERRQ(ierr);
-
+    ierr = PetscLogEventEnd(NEP_DerivativesEval,nep,A,0,0);CHKERRQ(ierr);
+    break;
   }
   PetscFunctionReturn(0);
 }
