@@ -19,11 +19,9 @@
            bidiagonalization", Elec. Trans. Numer. Anal. 31:68-85,
            2008.
 
-   Last update: Jun 2007
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -41,9 +39,18 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/svdimpl.h>                /*I "slepcsvd.h" I*/
-#include <slepc-private/ipimpl.h>
-#include <slepcblaslapack.h>
+#include <slepc/private/svdimpl.h>          /*I "slepcsvd.h" I*/
+
+static PetscBool  cited = PETSC_FALSE;
+static const char citation[] =
+  "@Article{slepc-svd,\n"
+  "   author = \"V. Hern{\\'a}ndez and J. E. Rom{\\'a}n and A. Tom{\\'a}s\",\n"
+  "   title = \"A robust and efficient parallel {SVD} solver based on restarted {Lanczos} bidiagonalization\",\n"
+  "   journal = \"Electron. Trans. Numer. Anal.\",\n"
+  "   volume = \"31\",\n"
+  "   pages = \"68--85\",\n"
+  "   year = \"2008\"\n"
+  "}\n";
 
 typedef struct {
   PetscBool oneside;
@@ -58,24 +65,11 @@ PetscErrorCode SVDSetUp_TRLanczos(SVD svd)
 
   PetscFunctionBegin;
   ierr = SVDMatGetSize(svd,NULL,&N);CHKERRQ(ierr);
-  if (svd->ncv) { /* ncv set */
-    if (svd->ncv<svd->nsv) SETERRQ(PetscObjectComm((PetscObject)svd),1,"The value of ncv must be at least nsv");
-  } else if (svd->mpd) { /* mpd set */
-    svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
-  } else { /* neither set: defaults depend on nsv being small or large */
-    if (svd->nsv<500) svd->ncv = PetscMin(N,PetscMax(2*svd->nsv,10));
-    else {
-      svd->mpd = 500;
-      svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
-    }
-  }
-  if (!svd->mpd) svd->mpd = svd->ncv;
+  ierr = SVDSetDimensions_Default(svd);CHKERRQ(ierr);
   if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(PetscObjectComm((PetscObject)svd),1,"The value of ncv must not be larger than nev+mpd");
   if (!svd->max_it) svd->max_it = PetscMax(N/svd->ncv,100);
-  if (svd->ncv!=svd->n) {
-    ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
-    ierr = PetscLogObjectParents(svd,svd->ncv,svd->U);CHKERRQ(ierr);
-  }
+  svd->leftbasis = PETSC_TRUE;
+  ierr = SVDAllocateSolution(svd,1);CHKERRQ(ierr);
   ierr = DSSetType(svd->ds,DSSVD);CHKERRQ(ierr);
   ierr = DSSetCompact(svd->ds,PETSC_TRUE);CHKERRQ(ierr);
   ierr = DSAllocate(svd->ds,svd->ncv);CHKERRQ(ierr);
@@ -84,145 +78,200 @@ PetscErrorCode SVDSetUp_TRLanczos(SVD svd)
 
 #undef __FUNCT__
 #define __FUNCT__ "SVDOneSideTRLanczosMGS"
-static PetscErrorCode SVDOneSideTRLanczosMGS(SVD svd,PetscReal *alpha,PetscReal *beta,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
+static PetscErrorCode SVDOneSideTRLanczosMGS(SVD svd,PetscReal *alpha,PetscReal *beta,BV V,BV U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
 {
   PetscErrorCode ierr;
   PetscReal      a,b;
   PetscInt       i,k=nconv+l;
+  Vec            ui,ui1,vi;
 
   PetscFunctionBegin;
-  ierr = SVDMatMult(svd,PETSC_FALSE,V[k],U[k]);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,k,&vi);CHKERRQ(ierr);
+  ierr = BVGetColumn(U,k,&ui);CHKERRQ(ierr);
+  ierr = SVDMatMult(svd,PETSC_FALSE,vi,ui);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,k,&vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(U,k,&ui);CHKERRQ(ierr);
   if (l>0) {
     for (i=0;i<l;i++) work[i]=beta[i+nconv];
-    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,work,U+nconv);CHKERRQ(ierr);
+    ierr = BVMultColumn(U,-1.0,1.0,k,work);CHKERRQ(ierr);
   }
-  ierr = IPNorm(svd->ip,U[k],&a);CHKERRQ(ierr);
-  ierr = VecScale(U[k],1.0/a);CHKERRQ(ierr);
+  ierr = BVNormColumn(U,k,NORM_2,&a);CHKERRQ(ierr);
+  ierr = BVScaleColumn(U,k,1.0/a);CHKERRQ(ierr);
   alpha[k] = a;
+
   for (i=k+1;i<n;i++) {
-    ierr = SVDMatMult(svd,PETSC_TRUE,U[i-1],V[i]);CHKERRQ(ierr);
-    ierr = IPOrthogonalize(svd->ip,0,NULL,i,NULL,V,V[i],work,&b,NULL);CHKERRQ(ierr);
-    ierr = VecScale(V[i],1.0/b);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = SVDMatMult(svd,PETSC_TRUE,ui1,vi);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = BVOrthogonalizeColumn(V,i,NULL,&b,NULL);CHKERRQ(ierr);
+    ierr = BVScaleColumn(V,i,1.0/b);CHKERRQ(ierr);
     beta[i-1] = b;
 
-    ierr = SVDMatMult(svd,PETSC_FALSE,V[i],U[i]);CHKERRQ(ierr);
-    ierr = VecAXPY(U[i],-b,U[i-1]);CHKERRQ(ierr);
-    ierr = IPNorm(svd->ip,U[i],&a);CHKERRQ(ierr);
-    ierr = VecScale(U[i],1.0/a);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i,&ui);CHKERRQ(ierr);
+    ierr = SVDMatMult(svd,PETSC_FALSE,vi,ui);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = VecAXPY(ui,-b,ui1);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i,&ui);CHKERRQ(ierr);
+    ierr = BVNormColumn(U,i,NORM_2,&a);CHKERRQ(ierr);
+    ierr = BVScaleColumn(U,i,1.0/a);CHKERRQ(ierr);
     alpha[i] = a;
   }
-  ierr = SVDMatMult(svd,PETSC_TRUE,U[n-1],v);CHKERRQ(ierr);
-  ierr = IPOrthogonalize(svd->ip,0,NULL,n,NULL,V,v,work,&b,NULL);CHKERRQ(ierr);
+
+  ierr = BVGetColumn(V,n,&vi);CHKERRQ(ierr);
+  ierr = BVGetColumn(U,n-1,&ui1);CHKERRQ(ierr);
+  ierr = SVDMatMult(svd,PETSC_TRUE,ui1,vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,n,&vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(U,n-1,&ui1);CHKERRQ(ierr);
+  ierr = BVOrthogonalizeColumn(V,n,NULL,&b,NULL);CHKERRQ(ierr);
   beta[n-1] = b;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "SVDOneSideTRLanczosCGS"
-static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal *beta,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
+#define __FUNCT__ "SVDOrthogonalizeCGS"
+/*
+  Custom CGS orthogonalization, preprocess after first orthogonalization
+*/
+static PetscErrorCode SVDOrthogonalizeCGS(BV V,PetscInt i,PetscScalar* h,PetscReal a,BVOrthogRefineType refine,PetscReal eta,PetscReal *norm)
 {
-  PetscErrorCode     ierr;
-  PetscReal          a,b,sum,onorm,eta;
-  PetscScalar        dot;
-  PetscInt           i,j,k=nconv+l;
-  IPOrthogRefineType refine;
+  PetscErrorCode ierr;
+  PetscReal      sum,onorm;
+  PetscScalar    dot;
+  PetscInt       j;
 
   PetscFunctionBegin;
-  ierr = SVDMatMult(svd,PETSC_FALSE,V[k],U[k]);CHKERRQ(ierr);
+  switch (refine) {
+  case BV_ORTHOG_REFINE_NEVER:
+    ierr = BVNormColumn(V,i,NORM_2,norm);CHKERRQ(ierr);
+    break;
+  case BV_ORTHOG_REFINE_ALWAYS:
+    ierr = BVSetActiveColumns(V,0,i);CHKERRQ(ierr);
+    ierr = BVDotColumn(V,i,h);CHKERRQ(ierr);
+    ierr = BVMultColumn(V,-1.0,1.0,i,h);CHKERRQ(ierr);
+    ierr = BVNormColumn(V,i,NORM_2,norm);CHKERRQ(ierr);
+    break;
+  case BV_ORTHOG_REFINE_IFNEEDED:
+    dot = h[i];
+    onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
+    sum = 0.0;
+    for (j=0;j<i;j++) {
+      sum += PetscRealPart(h[j] * PetscConj(h[j]));
+    }
+    *norm = PetscRealPart(dot)/(a*a) - sum;
+    if (*norm>0.0) *norm = PetscSqrtReal(*norm);
+    else {
+      ierr = BVNormColumn(V,i,NORM_2,norm);CHKERRQ(ierr);
+    }
+    if (*norm < eta*onorm) {
+      ierr = BVSetActiveColumns(V,0,i);CHKERRQ(ierr);
+      ierr = BVDotColumn(V,i,h);CHKERRQ(ierr);
+      ierr = BVMultColumn(V,-1.0,1.0,i,h);CHKERRQ(ierr);
+      ierr = BVNormColumn(V,i,NORM_2,norm);CHKERRQ(ierr);
+    }
+    break;
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "SVDOneSideTRLanczosCGS"
+static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal *beta,BV V,BV U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
+{
+  PetscErrorCode     ierr;
+  PetscReal          a,b,eta;
+  PetscInt           i,j,k=nconv+l;
+  Vec                ui,ui1,vi;
+  BVOrthogRefineType refine;
+
+  PetscFunctionBegin;
+  ierr = BVGetColumn(V,k,&vi);CHKERRQ(ierr);
+  ierr = BVGetColumn(U,k,&ui);CHKERRQ(ierr);
+  ierr = SVDMatMult(svd,PETSC_FALSE,vi,ui);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,k,&vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(U,k,&ui);CHKERRQ(ierr);
   if (l>0) {
     for (i=0;i<l;i++) work[i]=beta[i+nconv];
-    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,work,U+nconv);CHKERRQ(ierr);
+    ierr = BVMultColumn(U,-1.0,1.0,k,work);CHKERRQ(ierr);
   }
-  ierr = IPGetOrthogonalization(svd->ip,NULL,&refine,&eta);CHKERRQ(ierr);
+  ierr = BVGetOrthogonalization(V,NULL,&refine,&eta,NULL);CHKERRQ(ierr);
+
   for (i=k+1;i<n;i++) {
-    ierr = SVDMatMult(svd,PETSC_TRUE,U[i-1],V[i]);CHKERRQ(ierr);
-    ierr = IPNormBegin(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
-      ierr = IPInnerProductBegin(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = SVDMatMult(svd,PETSC_TRUE,ui1,vi);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = BVNormColumnBegin(U,i-1,NORM_2,&a);CHKERRQ(ierr);
+    if (refine == BV_ORTHOG_REFINE_IFNEEDED) {
+      ierr = BVSetActiveColumns(V,0,i+1);CHKERRQ(ierr);
+      ierr = BVGetColumn(V,i,&vi);CHKERRQ(ierr);
+      ierr = BVDotVecBegin(V,vi,work);CHKERRQ(ierr);
+    } else {
+      ierr = BVSetActiveColumns(V,0,i);CHKERRQ(ierr);
+      ierr = BVDotColumnBegin(V,i,work);CHKERRQ(ierr);
     }
-    ierr = IPMInnerProductBegin(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
-    ierr = IPNormEnd(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
-      ierr = IPInnerProductEnd(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
+    ierr = BVNormColumnEnd(U,i-1,NORM_2,&a);CHKERRQ(ierr);
+    if (refine == BV_ORTHOG_REFINE_IFNEEDED) {
+      ierr = BVDotVecEnd(V,vi,work);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(V,i,&vi);CHKERRQ(ierr);
+      ierr = BVSetActiveColumns(V,0,i);CHKERRQ(ierr);
+    } else {
+      ierr = BVDotColumnEnd(V,i,work);CHKERRQ(ierr);
     }
-    ierr = IPMInnerProductEnd(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
 
-    ierr = VecScale(U[i-1],1.0/a);CHKERRQ(ierr);
+    ierr = BVScaleColumn(U,i-1,1.0/a);CHKERRQ(ierr);
     for (j=0;j<i;j++) work[j] = work[j] / a;
-    ierr = SlepcVecMAXPBY(V[i],1.0/a,-1.0,i,work,V);CHKERRQ(ierr);
+    ierr = BVMultColumn(V,-1.0,1.0/a,i,work);CHKERRQ(ierr);
+    ierr = SVDOrthogonalizeCGS(V,i,work,a,refine,eta,&b);CHKERRQ(ierr);
+    ierr = BVScaleColumn(V,i,1.0/b);CHKERRQ(ierr);
+    if (PetscAbsReal(b)<10*PETSC_MACHINE_EPSILON) SETERRQ(PETSC_COMM_SELF,1,"Recurrence generated a zero vector; use a two-sided variant");
 
-    switch (refine) {
-    case IP_ORTHOG_REFINE_NEVER:
-      ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
-      break;
-    case IP_ORTHOG_REFINE_ALWAYS:
-      ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,i,NULL,V,V[i],work,NULL,&b);CHKERRQ(ierr);
-      break;
-    case IP_ORTHOG_REFINE_IFNEEDED:
-      onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
-      sum = 0.0;
-      for (j=0;j<i;j++) {
-        sum += PetscRealPart(work[j] * PetscConj(work[j]));
-      }
-      b = PetscRealPart(dot)/(a*a) - sum;
-      if (b>0.0) b = PetscSqrtReal(b);
-      else {
-        ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
-      }
-      if (b < eta*onorm) {
-        ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,i,NULL,V,V[i],work,NULL,&b);CHKERRQ(ierr);
-      }
-      break;
-    }
-
-    ierr = VecScale(V[i],1.0/b);CHKERRQ(ierr);
-
-    ierr = SVDMatMult(svd,PETSC_FALSE,V[i],U[i]);CHKERRQ(ierr);
-    ierr = VecAXPY(U[i],-b,U[i-1]);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i,&ui);CHKERRQ(ierr);
+    ierr = BVGetColumn(U,i-1,&ui1);CHKERRQ(ierr);
+    ierr = SVDMatMult(svd,PETSC_FALSE,vi,ui);CHKERRQ(ierr);
+    ierr = VecAXPY(ui,-b,ui1);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,i,&vi);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i,&ui);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(U,i-1,&ui1);CHKERRQ(ierr);
 
     alpha[i-1] = a;
     beta[i-1] = b;
   }
-  ierr = SVDMatMult(svd,PETSC_TRUE,U[n-1],v);CHKERRQ(ierr);
-  ierr = IPNormBegin(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
-    ierr = IPInnerProductBegin(svd->ip,v,v,&dot);CHKERRQ(ierr);
-  }
-  ierr = IPMInnerProductBegin(svd->ip,v,n,V,work);CHKERRQ(ierr);
-  ierr = IPNormEnd(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
-    ierr = IPInnerProductEnd(svd->ip,v,v,&dot);CHKERRQ(ierr);
-  }
-  ierr = IPMInnerProductEnd(svd->ip,v,n,V,work);CHKERRQ(ierr);
 
-  ierr = VecScale(U[n-1],1.0/a);CHKERRQ(ierr);
+  ierr = BVGetColumn(V,n,&vi);CHKERRQ(ierr);
+  ierr = BVGetColumn(U,n-1,&ui1);CHKERRQ(ierr);
+  ierr = SVDMatMult(svd,PETSC_TRUE,ui1,vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(V,n,&vi);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(U,n-1,&ui1);CHKERRQ(ierr);
+
+  ierr = BVNormColumnBegin(svd->U,n-1,NORM_2,&a);CHKERRQ(ierr);
+  if (refine == BV_ORTHOG_REFINE_IFNEEDED) {
+    ierr = BVSetActiveColumns(V,0,n+1);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,n,&vi);CHKERRQ(ierr);
+    ierr = BVDotVecBegin(V,vi,work);CHKERRQ(ierr);
+  } else {
+    ierr = BVSetActiveColumns(V,0,n);CHKERRQ(ierr);
+    ierr = BVDotColumnBegin(V,n,work);CHKERRQ(ierr);
+  }
+  ierr = BVNormColumnEnd(svd->U,n-1,NORM_2,&a);CHKERRQ(ierr);
+  if (refine == BV_ORTHOG_REFINE_IFNEEDED) {
+    ierr = BVDotVecEnd(V,vi,work);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,n,&vi);CHKERRQ(ierr);
+  } else {
+    ierr = BVDotColumnEnd(V,n,work);CHKERRQ(ierr);
+  }
+
+  ierr = BVScaleColumn(U,n-1,1.0/a);CHKERRQ(ierr);
   for (j=0;j<n;j++) work[j] = work[j] / a;
-  ierr = SlepcVecMAXPBY(v,1.0/a,-1.0,n,work,V);CHKERRQ(ierr);
-
-  switch (refine) {
-  case IP_ORTHOG_REFINE_NEVER:
-    ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
-    break;
-  case IP_ORTHOG_REFINE_ALWAYS:
-    ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,n,NULL,V,v,work,NULL,&b);CHKERRQ(ierr);
-    break;
-  case IP_ORTHOG_REFINE_IFNEEDED:
-    onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
-    sum = 0.0;
-    for (j=0;j<i;j++) {
-      sum += PetscRealPart(work[j] * PetscConj(work[j]));
-    }
-    b = PetscRealPart(dot)/(a*a) - sum;
-    if (b>0.0) b = PetscSqrtReal(b);
-    else {
-      ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
-    }
-    if (b < eta*onorm) {
-      ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,n,NULL,V,v,work,NULL,&b);CHKERRQ(ierr);
-    }
-    break;
-  }
-
+  ierr = BVMultColumn(V,-1.0,1.0/a,n,work);CHKERRQ(ierr);
+  ierr = SVDOrthogonalizeCGS(V,n,work,a,refine,eta,&b);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(V,nconv,n);CHKERRQ(ierr);
   alpha[n-1] = a;
   beta[n-1] = b;
   PetscFunctionReturn(0);
@@ -234,47 +283,52 @@ PetscErrorCode SVDSolve_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
-  PetscReal      *alpha,*beta,lastbeta,norm;
-  PetscScalar    *Q,*PT,*swork,*w;
-  PetscInt       i,k,l,nv,ld,off;
-  Vec            v;
+  PetscReal      *alpha,*beta,lastbeta,norm,resnorm;
+  PetscScalar    *Q,*swork=NULL,*w;
+  PetscInt       i,k,l,nv,ld;
+  Mat            U,VT;
   PetscBool      conv;
-  IPOrthogType   orthog;
+  BVOrthogType   orthog;
 
   PetscFunctionBegin;
+  ierr = PetscCitationsRegister(citation,&cited);CHKERRQ(ierr);
   /* allocate working space */
   ierr = DSGetLeadingDimension(svd->ds,&ld);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscScalar)*ld,&w);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscScalar)*svd->n,&swork);CHKERRQ(ierr);
-  ierr = VecDuplicate(svd->V[0],&v);CHKERRQ(ierr);
-  ierr = IPGetOrthogonalization(svd->ip,&orthog,NULL,NULL);CHKERRQ(ierr);
+  ierr = BVGetOrthogonalization(svd->V,&orthog,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ld,&w);CHKERRQ(ierr);
+  if (lanczos->oneside) {
+    ierr = PetscMalloc1(svd->ncv+1,&swork);CHKERRQ(ierr);
+  }
 
   /* normalize start vector */
   if (!svd->nini) {
-    ierr = SlepcVecSetRandom(svd->V[0],svd->rand);CHKERRQ(ierr);
+    ierr = BVSetRandomColumn(svd->V,0);CHKERRQ(ierr);
+    ierr = BVNormColumn(svd->V,0,NORM_2,&norm);CHKERRQ(ierr);
+    ierr = BVScaleColumn(svd->V,0,1.0/norm);CHKERRQ(ierr);
   }
-  ierr = VecNormalize(svd->V[0],&norm);CHKERRQ(ierr);
 
   l = 0;
   while (svd->reason == SVD_CONVERGED_ITERATING) {
     svd->its++;
 
     /* inner loop */
-    nv = PetscMin(svd->nconv+svd->mpd,svd->n);
+    nv = PetscMin(svd->nconv+svd->mpd,svd->ncv);
+    ierr = BVSetActiveColumns(svd->V,svd->nconv,nv);CHKERRQ(ierr);
+    ierr = BVSetActiveColumns(svd->U,svd->nconv,nv);CHKERRQ(ierr);
     ierr = DSGetArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
     beta = alpha + ld;
     if (lanczos->oneside) {
-      if (orthog == IP_ORTHOG_MGS) {
-        ierr = SVDOneSideTRLanczosMGS(svd,alpha,beta,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
+      if (orthog == BV_ORTHOG_MGS) {
+        ierr = SVDOneSideTRLanczosMGS(svd,alpha,beta,svd->V,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
       } else {
-        ierr = SVDOneSideTRLanczosCGS(svd,alpha,beta,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
+        ierr = SVDOneSideTRLanczosCGS(svd,alpha,beta,svd->V,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
       }
     } else {
-      ierr = SVDTwoSideLanczos(svd,alpha,beta,svd->V,v,svd->U,svd->nconv+l,nv,swork);CHKERRQ(ierr);
+      ierr = SVDTwoSideLanczos(svd,alpha,beta,svd->V,svd->U,svd->nconv+l,nv);CHKERRQ(ierr);
     }
     lastbeta = beta[nv-1];
     ierr = DSRestoreArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
-    ierr = VecScale(v,1.0/lastbeta);CHKERRQ(ierr);
+    ierr = BVScaleColumn(svd->V,nv,1.0/lastbeta);CHKERRQ(ierr);
 
     /* compute SVD of general matrix */
     ierr = DSSetDimensions(svd->ds,nv,nv,svd->nconv,svd->nconv+l);CHKERRQ(ierr);
@@ -295,32 +349,32 @@ PetscErrorCode SVDSolve_TRLanczos(SVD svd)
     for (i=svd->nconv;i<nv;i++) {
       svd->sigma[i] = PetscRealPart(w[i]);
       beta[i] = PetscRealPart(Q[nv-1+i*ld])*lastbeta;
-      svd->errest[i] = PetscAbsScalar(beta[i]);
-      if (svd->sigma[i] > svd->tol) svd->errest[i] /= svd->sigma[i];
+      resnorm = PetscAbsReal(beta[i]);
+      ierr = (*svd->converged)(svd,svd->sigma[i],resnorm,&svd->errest[i],svd->convergedctx);CHKERRQ(ierr);
       if (conv) {
         if (svd->errest[i] < svd->tol) k++;
         else conv = PETSC_FALSE;
       }
     }
     ierr = DSRestoreArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
+    ierr = DSRestoreArray(svd->ds,DS_MAT_U,&Q);CHKERRQ(ierr);
 
     /* check convergence and update l */
-    if (svd->its >= svd->max_it) svd->reason = SVD_DIVERGED_ITS;
-    if (svd->nconv+k >= svd->nsv) svd->reason = SVD_CONVERGED_TOL;
+    ierr = (*svd->stopping)(svd,svd->its,svd->max_it,svd->nconv+k,svd->nsv,&svd->reason,svd->stoppingctx);CHKERRQ(ierr);
     if (svd->reason != SVD_CONVERGED_ITERATING) l = 0;
     else l = PetscMax((nv-svd->nconv-k)/2,0);
 
     /* compute converged singular vectors and restart vectors */
-    off = svd->nconv+svd->nconv*ld;
-    ierr = DSGetArray(svd->ds,DS_MAT_VT,&PT);CHKERRQ(ierr);
-    ierr = SlepcUpdateVectors(nv-svd->nconv,svd->V+svd->nconv,0,k+l,PT+off,ld,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = SlepcUpdateVectors(nv-svd->nconv,svd->U+svd->nconv,0,k+l,Q+off,ld,PETSC_FALSE);CHKERRQ(ierr);
-    ierr = DSRestoreArray(svd->ds,DS_MAT_VT,&PT);CHKERRQ(ierr);
-    ierr = DSRestoreArray(svd->ds,DS_MAT_U,&Q);CHKERRQ(ierr);
+    ierr = DSGetMat(svd->ds,DS_MAT_VT,&VT);CHKERRQ(ierr);
+    ierr = BVMultInPlaceTranspose(svd->V,VT,svd->nconv,svd->nconv+k+l);CHKERRQ(ierr);
+    ierr = MatDestroy(&VT);CHKERRQ(ierr);
+    ierr = DSGetMat(svd->ds,DS_MAT_U,&U);CHKERRQ(ierr);
+    ierr = BVMultInPlace(svd->U,U,svd->nconv,svd->nconv+k+l);CHKERRQ(ierr);
+    ierr = MatDestroy(&U);CHKERRQ(ierr);
 
     /* copy the last vector to be the next initial vector */
     if (svd->reason == SVD_CONVERGED_ITERATING) {
-      ierr = VecCopy(v,svd->V[svd->nconv+k+l]);CHKERRQ(ierr);
+      ierr = BVCopyColumn(svd->V,nv,svd->nconv+k+l);CHKERRQ(ierr);
     }
 
     svd->nconv += k;
@@ -330,28 +384,27 @@ PetscErrorCode SVDSolve_TRLanczos(SVD svd)
   /* orthonormalize U columns in one side method */
   if (lanczos->oneside) {
     for (i=0;i<svd->nconv;i++) {
-      ierr = IPOrthogonalize(svd->ip,0,NULL,i,NULL,svd->U,svd->U[i],NULL,&norm,NULL);CHKERRQ(ierr);
-      ierr = VecScale(svd->U[i],1.0/norm);CHKERRQ(ierr);
+      ierr = BVOrthogonalizeColumn(svd->U,i,NULL,&norm,NULL);CHKERRQ(ierr);
+      ierr = BVScaleColumn(svd->U,i,1.0/norm);CHKERRQ(ierr);
     }
   }
 
   /* free working space */
-  ierr = VecDestroy(&v);CHKERRQ(ierr);
   ierr = PetscFree(w);CHKERRQ(ierr);
-  ierr = PetscFree(swork);CHKERRQ(ierr);
+  if (swork) { ierr = PetscFree(swork);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "SVDSetFromOptions_TRLanczos"
-PetscErrorCode SVDSetFromOptions_TRLanczos(SVD svd)
+PetscErrorCode SVDSetFromOptions_TRLanczos(PetscOptionItems *PetscOptionsObject,SVD svd)
 {
   PetscErrorCode ierr;
   PetscBool      set,val;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("SVD TRLanczos Options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"SVD TRLanczos Options");CHKERRQ(ierr);
   ierr = PetscOptionsBool("-svd_trlanczos_oneside","Lanczos one-side reorthogonalization","SVDTRLanczosSetOneSide",lanczos->oneside,&val,&set);CHKERRQ(ierr);
   if (set) {
     ierr = SVDTRLanczosSetOneSide(svd,val);CHKERRQ(ierr);
@@ -406,6 +459,17 @@ PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscBool oneside)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "SVDTRLanczosGetOneSide_TRLanczos"
+static PetscErrorCode SVDTRLanczosGetOneSide_TRLanczos(SVD svd,PetscBool *oneside)
+{
+  SVD_TRLANCZOS *lanczos = (SVD_TRLANCZOS*)svd->data;
+
+  PetscFunctionBegin;
+  *oneside = lanczos->oneside;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__
 #define __FUNCT__ "SVDTRLanczosGetOneSide"
 /*@
@@ -431,29 +495,7 @@ PetscErrorCode SVDTRLanczosGetOneSide(SVD svd,PetscBool *oneside)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(oneside,2);
-  ierr = PetscTryMethod(svd,"SVDTRLanczosGetOneSide_C",(SVD,PetscBool*),(svd,oneside));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SVDTRLanczosGetOneSide_TRLanczos"
-static PetscErrorCode SVDTRLanczosGetOneSide_TRLanczos(SVD svd,PetscBool *oneside)
-{
-  SVD_TRLANCZOS    *lanczos = (SVD_TRLANCZOS*)svd->data;
-
-  PetscFunctionBegin;
-  *oneside = lanczos->oneside;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SVDReset_TRLanczos"
-PetscErrorCode SVDReset_TRLanczos(SVD svd)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr);
+  ierr = PetscUseMethod(svd,"SVDTRLanczosGetOneSide_C",(SVD,PetscBool*),(svd,oneside));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -476,9 +518,13 @@ PetscErrorCode SVDView_TRLanczos(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscViewerASCIIPrintf(viewer,"  TRLanczos: %s-sided reorthogonalization\n",lanczos->oneside? "one": "two");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  TRLanczos: %s-sided reorthogonalization\n",lanczos->oneside? "one": "two");CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -487,13 +533,15 @@ PetscErrorCode SVDView_TRLanczos(SVD svd,PetscViewer viewer)
 PETSC_EXTERN PetscErrorCode SVDCreate_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
+  SVD_TRLANCZOS  *ctx;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(svd,SVD_TRLANCZOS,&svd->data);CHKERRQ(ierr);
+  ierr = PetscNewLog(svd,&ctx);CHKERRQ(ierr);
+  svd->data = (void*)ctx;
+
   svd->ops->setup          = SVDSetUp_TRLanczos;
   svd->ops->solve          = SVDSolve_TRLanczos;
   svd->ops->destroy        = SVDDestroy_TRLanczos;
-  svd->ops->reset          = SVDReset_TRLanczos;
   svd->ops->setfromoptions = SVDSetFromOptions_TRLanczos;
   svd->ops->view           = SVDView_TRLanczos;
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDTRLanczosSetOneSide_C",SVDTRLanczosSetOneSide_TRLanczos);CHKERRQ(ierr);
