@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -19,7 +19,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/svdimpl.h>       /*I "slepcsvd.h" I*/
+#include <slepc/private/svdimpl.h>       /*I "slepcsvd.h" I*/
 
 PETSC_EXTERN PetscErrorCode SVDCreate_Cross(SVD);
 PETSC_EXTERN PetscErrorCode SVDCreate_Cyclic(SVD);
@@ -43,6 +43,7 @@ PetscErrorCode SVDRegisterAll(void)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  if (SVDRegisterAllCalled) PetscFunctionReturn(0);
   SVDRegisterAllCalled = PETSC_TRUE;
   ierr = SVDRegister(SVDCROSS,SVDCreate_Cross);CHKERRQ(ierr);
   ierr = SVDRegister(SVDCYCLIC,SVDCreate_Cyclic);CHKERRQ(ierr);
@@ -51,3 +52,4 @@ PetscErrorCode SVDRegisterAll(void)
   ierr = SVDRegister(SVDTRLANCZOS,SVDCreate_TRLanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
