@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -19,8 +19,53 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/slepcimpl.h>            /*I "slepcsys.h" I*/
-#include <petsc-private/matimpl.h>
+#include <slepc/private/slepcimpl.h>            /*I "slepcsys.h" I*/
+
+#undef __FUNCT__
+#define __FUNCT__ "SlepcVecNormalize"
+/*@C
+   SlepcVecNormalize - Normalizes a possibly complex vector by the 2-norm.
+
+   Collective on Vec
+
+   Input parameters:
++  xr - the real part of the vector (overwritten on output)
+.  xi - the imaginary part of the vector (not referenced if iscomplex is false)
+-  iscomplex - a flag indicating if the vector is complex
+
+   Output parameter:
+.  norm - the vector norm before normalization (can be set to NULL)
+
+   Level: developer
+@*/
+PetscErrorCode SlepcVecNormalize(Vec xr,Vec xi,PetscBool iscomplex,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscReal      normr,normi,alpha;
+#endif
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(xr,VEC_CLASSID,1);
+#if !defined(PETSC_USE_COMPLEX)
+  if (iscomplex) {
+    PetscValidHeaderSpecific(xi,VEC_CLASSID,2);
+    ierr = VecNormBegin(xr,NORM_2,&normr);CHKERRQ(ierr);
+    ierr = VecNormBegin(xi,NORM_2,&normi);CHKERRQ(ierr);
+    ierr = VecNormEnd(xr,NORM_2,&normr);CHKERRQ(ierr);
+    ierr = VecNormEnd(xi,NORM_2,&normi);CHKERRQ(ierr);
+    alpha = SlepcAbsEigenvalue(normr,normi);
+    if (norm) *norm = alpha;
+    alpha = 1.0 / alpha;
+    ierr = VecScale(xr,alpha);CHKERRQ(ierr);
+    ierr = VecScale(xi,alpha);CHKERRQ(ierr);
+  } else
+#endif
+  {
+    ierr = VecNormalize(xr,norm);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__
 #define __FUNCT__ "SlepcMatConvertSeqDense"
@@ -41,6 +86,7 @@ PetscErrorCode SlepcMatConvertSeqDense(Mat mat,Mat *newmat)
   PetscErrorCode ierr;
   PetscInt       m,n;
   PetscMPIInt    size;
+  PetscBool      flg;
   Mat            *M;
   IS             isrow,iscol;
 
@@ -49,7 +95,8 @@ PetscErrorCode SlepcMatConvertSeqDense(Mat mat,Mat *newmat)
   PetscValidPointer(newmat,2);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)mat),&size);CHKERRQ(ierr);
   if (size > 1) {
-    if (!mat->ops->getsubmatrices) SETERRQ1(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"Mat type %s",((PetscObject)mat)->type_name);
+    ierr = MatHasOperation(mat,MATOP_GET_SUBMATRICES,&flg);CHKERRQ(ierr);
+    if (!flg) SETERRQ1(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"Mat type %s",((PetscObject)mat)->type_name);
 
     /* assemble full matrix on every processor */
     ierr = MatGetSize(mat,&m,&n);CHKERRQ(ierr);
@@ -88,8 +135,7 @@ static PetscErrorCode SlepcMatTile_SeqAIJ(PetscScalar a,Mat A,PetscScalar b,Mat
   ierr = MatGetSize(A,&M1,&N1);CHKERRQ(ierr);
   ierr = MatGetSize(D,&M2,&N2);CHKERRQ(ierr);
 
-  ierr = PetscMalloc((M1+M2)*sizeof(PetscInt),&nnz);CHKERRQ(ierr);
-  ierr = PetscMemzero(nnz,(M1+M2)*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(M1+M2,&nnz);CHKERRQ(ierr);
   /* Preallocate for A */
   if (a!=0.0) {
     for (i=0;i<M1;i++) {
@@ -125,8 +171,7 @@ static PetscErrorCode SlepcMatTile_SeqAIJ(PetscScalar a,Mat A,PetscScalar b,Mat
   ierr = MatSeqAIJSetPreallocation(G,0,nnz);CHKERRQ(ierr);
   ierr = PetscFree(nnz);CHKERRQ(ierr);
 
-  ierr = PetscMalloc(sizeof(PetscScalar)*PetscMax(N1,N2),&buf);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscInt)*PetscMax(N1,N2),&scols);CHKERRQ(ierr);
+  ierr = PetscMalloc2(PetscMax(N1,N2),&buf,PetscMax(N1,N2),&scols);CHKERRQ(ierr);
   /* Transfer A */
   if (a!=0.0) {
     for (i=0;i<M1;i++) {
@@ -179,8 +224,7 @@ static PetscErrorCode SlepcMatTile_SeqAIJ(PetscScalar a,Mat A,PetscScalar b,Mat
       ierr = MatRestoreRow(D,i,&ncols,&cols,&vals);CHKERRQ(ierr);
     }
   }
-  ierr = PetscFree(buf);CHKERRQ(ierr);
-  ierr = PetscFree(scols);CHKERRQ(ierr);
+  ierr = PetscFree2(buf,scols);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -203,21 +247,17 @@ static PetscErrorCode SlepcMatTile_MPIAIJ(PetscScalar a,Mat A,PetscScalar b,Mat
   ierr = MatGetLocalSize(D,&m2,&n2);CHKERRQ(ierr);
 
   /* Create mappings */
-  MPI_Comm_size(PetscObjectComm((PetscObject)G),&np);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)G),&np);CHKERRQ(ierr);
   ierr = MatGetOwnershipRangesColumn(A,&mapptr1);CHKERRQ(ierr);
   ierr = MatGetOwnershipRangesColumn(B,&mapptr2);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscInt)*N1,&map1);CHKERRQ(ierr);
+  ierr = PetscMalloc4(PetscMax(N1,N2),&buf,PetscMax(N1,N2),&scols,N1,&map1,N2,&map2);CHKERRQ(ierr);
   for (p=0;p<np;p++) {
     for (i=mapptr1[p];i<mapptr1[p+1];i++) map1[i] = i+mapptr2[p];
   }
-  ierr = PetscMalloc(sizeof(PetscInt)*N2,&map2);CHKERRQ(ierr);
   for (p=0;p<np;p++) {
     for (i=mapptr2[p];i<mapptr2[p+1];i++) map2[i] = i+mapptr1[p+1];
   }
 
-  ierr = PetscMalloc(sizeof(PetscScalar)*PetscMax(N1,N2),&buf);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscInt)*PetscMax(N1,N2),&scols);CHKERRQ(ierr);
-
   ierr = MatPreallocateInitialize(PetscObjectComm((PetscObject)G),m1+m2,n1+n2,dnz,onz);CHKERRQ(ierr);
   ierr = MatGetOwnershipRange(G,&gstart,NULL);CHKERRQ(ierr);
   /* Preallocate for A */
@@ -323,10 +363,7 @@ static PetscErrorCode SlepcMatTile_MPIAIJ(PetscScalar a,Mat A,PetscScalar b,Mat
       ierr = MatRestoreRow(D,i+start,&ncols,&cols,&vals);CHKERRQ(ierr);
     }
   }
-  ierr = PetscFree(buf);CHKERRQ(ierr);
-  ierr = PetscFree(scols);CHKERRQ(ierr);
-  ierr = PetscFree(map1);CHKERRQ(ierr);
-  ierr = PetscFree(map2);CHKERRQ(ierr);
+  ierr = PetscFree4(buf,scols,map1,map2);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -423,7 +460,7 @@ PetscErrorCode SlepcMatTile(PetscScalar a,Mat A,PetscScalar b,Mat B,PetscScalar
 
 #undef __FUNCT__
 #define __FUNCT__ "SlepcCheckOrthogonality"
-/*@
+/*@C
    SlepcCheckOrthogonality - Checks (or prints) the level of orthogonality
    of a set of vectors.
 
@@ -472,7 +509,7 @@ PetscErrorCode SlepcCheckOrthogonality(Vec *V,PetscInt nv,Vec *W,PetscInt nw,Mat
     if (!isascii) PetscFunctionReturn(0);
   }
 
-  ierr = PetscMalloc(nv*sizeof(PetscScalar),&vals);CHKERRQ(ierr);
+  ierr = PetscMalloc1(nv,&vals);CHKERRQ(ierr);
   if (B) {
     ierr = VecDuplicate(V[0],&w);CHKERRQ(ierr);
   }
@@ -493,9 +530,9 @@ PetscErrorCode SlepcCheckOrthogonality(Vec *V,PetscInt nv,Vec *W,PetscInt nw,Mat
       if (lev) *lev = PetscMax(*lev,PetscAbsScalar((j==i)? (vals[j]-1.0): vals[j]));
       else {
 #if !defined(PETSC_USE_COMPLEX)
-        ierr = PetscViewerASCIIPrintf(viewer," %12G  ",vals[j]);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer," %12g  ",(double)vals[j]);CHKERRQ(ierr);
 #else
-        ierr = PetscViewerASCIIPrintf(viewer," %12G%+12Gi ",PetscRealPart(vals[j]),PetscImaginaryPart(vals[j]));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer," %12g%+12gi ",(double)PetscRealPart(vals[j]),(double)PetscImaginaryPart(vals[j]));CHKERRQ(ierr);
 #endif
       }
     }
@@ -508,12 +545,57 @@ PetscErrorCode SlepcCheckOrthogonality(Vec *V,PetscInt nv,Vec *W,PetscInt nw,Mat
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "SlepcConvMonitorCreate"
+/*@C
+   SlepcConvMonitorCreate - Creates a SlepcConvMonitor context.
+
+   Collective on PetscViewer
+
+   Input Parameters:
++  viewer - the viewer where the monitor must send data
+-  format - the format 
+
+   Output Parameter:
+.  ctx - the created context
+
+   Notes:
+   The created context is used for EPS, SVD, PEP, and NEP monitor functions that just
+   print the iteration numbers at which convergence takes place (XXXMonitorConverged).
+
+   This function increases the reference count of the viewer so you can destroy the
+   viewer object after this call.
+
+   Level: developer
+
+.seealso: SlepcConvMonitorDestroy()
+@*/
+PetscErrorCode SlepcConvMonitorCreate(PetscViewer viewer,PetscViewerFormat format,SlepcConvMonitor *ctx)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectReference((PetscObject)viewer);CHKERRQ(ierr);
+  ierr = PetscNew(ctx);CHKERRQ(ierr);
+  (*ctx)->viewer = viewer;
+  (*ctx)->format = format;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__
 #define __FUNCT__ "SlepcConvMonitorDestroy"
-/*
-  Clean up context used in monitors of type XXXMonitorConverged.
-  This function is shared by EPS, SVD, QEP
-*/
+/*@C
+   SlepcConvMonitorDestroy - Destroys a SlepcConvMonitor context.
+
+   Collective on PetscViewer
+
+   Input Parameters:
+.  ctx - the SlepcConvMonitor context to be destroyed.
+
+   Level: developer
+
+.seealso: SlepcConvMonitorCreate()
+@*/
 PetscErrorCode SlepcConvMonitorDestroy(SlepcConvMonitor *ctx)
 {
   PetscErrorCode ierr;
@@ -525,190 +607,6 @@ PetscErrorCode SlepcConvMonitorDestroy(SlepcConvMonitor *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareLargestMagnitude"
-PetscErrorCode SlepcCompareLargestMagnitude(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-  a = SlepcAbsEigenvalue(ar,ai);
-  b = SlepcAbsEigenvalue(br,bi);
-  if (a<b) *result = 1;
-  else if (a>b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareSmallestMagnitude"
-PetscErrorCode SlepcCompareSmallestMagnitude(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-  a = SlepcAbsEigenvalue(ar,ai);
-  b = SlepcAbsEigenvalue(br,bi);
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareLargestReal"
-PetscErrorCode SlepcCompareLargestReal(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-  a = PetscRealPart(ar);
-  b = PetscRealPart(br);
-  if (a<b) *result = 1;
-  else if (a>b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareSmallestReal"
-PetscErrorCode SlepcCompareSmallestReal(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-  a = PetscRealPart(ar);
-  b = PetscRealPart(br);
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareLargestImaginary"
-PetscErrorCode SlepcCompareLargestImaginary(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-#if defined(PETSC_USE_COMPLEX)
-  a = PetscImaginaryPart(ar);
-  b = PetscImaginaryPart(br);
-#else
-  a = PetscAbsReal(ai);
-  b = PetscAbsReal(bi);
-#endif
-  if (a<b) *result = 1;
-  else if (a>b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareSmallestImaginary"
-PetscErrorCode SlepcCompareSmallestImaginary(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-
-  PetscFunctionBegin;
-#if defined(PETSC_USE_COMPLEX)
-  a = PetscImaginaryPart(ar);
-  b = PetscImaginaryPart(br);
-#else
-  a = PetscAbsReal(ai);
-  b = PetscAbsReal(bi);
-#endif
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareTargetMagnitude"
-PetscErrorCode SlepcCompareTargetMagnitude(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal   a,b;
-  PetscScalar *target = (PetscScalar*)ctx;
-
-  PetscFunctionBegin;
-  /* complex target only allowed if scalartype=complex */
-  a = SlepcAbsEigenvalue(ar-(*target),ai);
-  b = SlepcAbsEigenvalue(br-(*target),bi);
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareTargetReal"
-PetscErrorCode SlepcCompareTargetReal(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal   a,b;
-  PetscScalar *target = (PetscScalar*)ctx;
-
-  PetscFunctionBegin;
-  a = PetscAbsReal(PetscRealPart(ar-(*target)));
-  b = PetscAbsReal(PetscRealPart(br-(*target)));
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareTargetImaginary"
-PetscErrorCode SlepcCompareTargetImaginary(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal   a,b;
-#if defined(PETSC_USE_COMPLEX)
-  PetscScalar *target = (PetscScalar*)ctx;
-#endif
-
-  PetscFunctionBegin;
-#if !defined(PETSC_USE_COMPLEX)
-  /* complex target only allowed if scalartype=complex */
-  a = PetscAbsReal(ai);
-  b = PetscAbsReal(bi);
-#else
-  a = PetscAbsReal(PetscImaginaryPart(ar-(*target)));
-  b = PetscAbsReal(PetscImaginaryPart(br-(*target)));
-#endif
-  if (a>b) *result = 1;
-  else if (a<b) *result = -1;
-  else *result = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SlepcCompareSmallestPosReal"
-/*
-   Used in the SVD for computing smallest singular values
-   from the cyclic matrix.
-*/
-PetscErrorCode SlepcCompareSmallestPosReal(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result,void *ctx)
-{
-  PetscReal a,b;
-  PetscBool aisright,bisright;
-
-  PetscFunctionBegin;
-  if (PetscRealPart(ar)>0.0) aisright = PETSC_TRUE;
-  else aisright = PETSC_FALSE;
-  if (PetscRealPart(br)>0.0) bisright = PETSC_TRUE;
-  else bisright = PETSC_FALSE;
-  if (aisright == bisright) { /* same sign */
-    a = SlepcAbsEigenvalue(ar,ai);
-    b = SlepcAbsEigenvalue(br,bi);
-    if (a>b) *result = 1;
-    else if (a<b) *result = -1;
-    else *result = 0;
-  } else if (aisright && !bisright) *result = -1; /* 'a' is on the right */
-  else *result = 1;  /* 'b' is on the right */
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__
 #define __FUNCT__ "SlepcBasisReference_Private"
 /*
@@ -723,7 +621,7 @@ PetscErrorCode SlepcBasisReference_Private(PetscInt n,Vec *V,PetscInt *m,Vec **W
   PetscFunctionBegin;
   ierr = SlepcBasisDestroy_Private(m,W);CHKERRQ(ierr);
   if (n>0) {
-    ierr = PetscMalloc(n*sizeof(Vec),W);CHKERRQ(ierr);
+    ierr = PetscMalloc1(n,W);CHKERRQ(ierr);
     for (i=0;i<n;i++) {
       ierr = PetscObjectReference((PetscObject)V[i]);CHKERRQ(ierr);
       (*W)[i] = V[i];
@@ -782,24 +680,24 @@ PetscErrorCode SlepcSNPrintfScalar(char *str,size_t len,PetscScalar val,PetscBoo
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
   if (exp) {
-    ierr = PetscSNPrintf(str,len,"%+G",val);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(str,len,"%+g",(double)val);CHKERRQ(ierr);
   } else {
-    ierr = PetscSNPrintf(str,len,"%G",val);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(str,len,"%g",(double)val);CHKERRQ(ierr);
   }
 #else
   re = PetscRealPart(val);
   im = PetscImaginaryPart(val);
   if (im!=0.0) {
     if (exp) {
-      ierr = PetscSNPrintf(str,len,"+(%G%+G i)",re,im);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(str,len,"+(%g%+gi)",(double)re,(double)im);CHKERRQ(ierr);
     } else {
-      ierr = PetscSNPrintf(str,len,"%G%+G i",re,im);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(str,len,"%g%+gi",(double)re,(double)im);CHKERRQ(ierr);
     }
   } else {
     if (exp) {
-      ierr = PetscSNPrintf(str,len,"%+G",re,im);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(str,len,"%+g",(double)re);CHKERRQ(ierr);
     } else {
-      ierr = PetscSNPrintf(str,len,"%G",re,im);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(str,len,"%g",(double)re);CHKERRQ(ierr);
     }
   }
 #endif
